\ Analoge datalogger met uitvoer naar RS232 en LEDS: 872 bytes. 

90S2313                                 \ Compileer voor AT90S2313
NEEDS TARGET.FRT

\ ByteForth RS232 woorden

UBRR  SFR RS232-SNELHEID
UCR   SFR RS232-MODE
UDR   SFR RS232-BUFFER

USR 5 BIT-SFR EMIT? ( -- flag )         \ True als zender nog bezig is
USR 7 BIT-SFR KEY?  ( -- flag )         \ True als ontvang buffer vol is

: RS232-SETUP   ( -- )                  \ Installeer RS232
    103 TO RS232-SNELHEID               \ Selecteer 2400 baud
    $18 TO RS232-MODE                   \ RS232 ontvanger en zender aan
    ;                                   \ Geen interrupt en 8 bits data.

: EMIT          ( char -- )             \ Stuur karakter via RS232
    BEGIN  EMIT? UNTIL
    TO RS232-BUFFER
    ;

: TYPE          ( $addr u -- )          \ Stuur ROM string via RS232
    ?DUP IF
        FOR
            COUNT EMIT
        NEXT
    THEN
    2DROP
    ;

: KEY           ( -- char )             \ Ontvang karakter via RS232
    BEGIN  KEY? UNTIL
    RS232-BUFFER
    ;


: RTYPE         ( addr u -- )           \ Stuur RAM string via RS232
    ?DUP IF
        FOR
            @+ EMIT
        NEXT
    THEN
    2DROP
    ;

: CR        ^M EMIT  ^J EMIT ;      ( -- )
: SPACE     BL EMIT ;                     ( -- )
: PAGE      ^L EMIT ;                     ( -- )

\ Einde RS232

\ Getal uitvoer

ATOM UM/MOD                             \ Importeer enkele macros als woord
ATOM INLINE$
VARIABLE BASE                           \ Getal conversie basis
VARIABLE HLD                            \ Werk adr. in getal conv. buffer
#10 CONSTANT NUM                        \ Length of number conversion buffer
NUM VARIABLES CONV                      \ Number string buffer
ADR CONV 2CONSTANT NUMSTART

: M/MOD     ( d1 u1 -- u2 d2 )          \ Deel dubbel met teken door single
    >R  0  R@ UM/MOD  R> SWAP >R  UM/MOD R> \ zonder teken.
    ;

\ Getal uitvoer primitieven

\ Zet karakter char in de uitvoer buffer
: HOLD      DECR HLD  FROM HLD TO CONV ;                ( char -- )

\ Initialiseer de uitvoer buffer
: <#        NUM TO HLD ;                                ( -- )

\ Beeindig getal conversie, geef addr en lengte u van de string in de buffer
: #>        ( d -- addr u )
    2DROP  NUMSTART FROM HLD M+  NUM FROM HLD - 
    ;

\ Zet een digit van d1 in de uitvoer buffer, d2 blijft over
: #         FROM BASE M/MOD  ROT 9 OVER <               ( d1 -- d2 )
            IF  7 +  THEN  [CHAR] 0 +  HOLD ;

\ Getal uitvoer woorden

\ Druk getal u af met digits en twee spaties er achter
: .DATA     0 <# # # # #>  RTYPE SPACE SPACE ;          ( u -- )

\ Zet getal conversie basis decimaal
: DECIMAL   #10 TO BASE ;                               ( -- )

\ Einde getal uitvoer

\ Data logger die elke seconde een meting bewaard en na 20 metingen
\ de data in een twee blokken via de RS232 verstuurt.

PORTB SFR LEDS                          \ Poort-B met 8 leds

PORTD 4 BIT-SFR ADC-KLOK                \ Klokpuls invoer PD.4
PORTD 5 BIT-SFR ADC-DATA                \ Data uitvoer PD.5
PORTD 6 BIT-SFR ADC-SELECTEER           \ Activeer invoer PD.6

: ADC               ( -- u )            \ Doe een AD-omzetting
    CLEAR ADC-KLOK                      \ Stop lezen van analoge invoer
    SET ADC-SELECTEER                   \ Start AD-omzetting
    24 FOR NEXT                         \ Conversie duurt min. 17 us 
    CLEAR ADC-SELECTEER                 \ Start uitlezen
    NOOP NOOP NOOP  FROM ADC-DATA 1 AND \ Lees databit 7
    7 FOR                               \ Lees resterende 7 bits
        SET ADC-KLOK
        CLEAR ADC-KLOK                  \ Schuif volgend bit naar buiten
        2*  FROM ADC-DATA  1 AND  OR    \ Schaal data in, lees volgend bit
    NEXT
    SET ADC-KLOK                        \ Blijf analoge invoer lezen
    ;

: 100MS  FOR  100 MS  NEXT ;            \ Wacht u maal 100 milliseconden

: NAAR-LEDS     ( x -- )                \ Zet x op leds
    INVERT  TO LEDS                     \ Inverteer x en stuur leds aan
    ;

: KNIPPER       ( -- )                  \ Visualiseer opstarten
    -1 NAAR-LEDS  250 MS                \ Alle leds aan
    0 NAAR-LEDS   250 MS                \ Alle leds uit
    ;

: BEELD-AF      ( u -- )                \ Toon met VU- of thermometer methode
    0 32 UM/MOD                         \ Schaal waarde in voor 8 leds
    SWAP 15 > IF  1+  THEN              \ Rond af naar hogere waarde
    DUP IF                              \ Uitkomst groter dan nul ?
        1 SWAP 1- LSHIFT                \ Ja, bepaal positie van led
\       DUP 1- OR                       \ Vul leds daaronder in
    THEN
    NAAR-LEDS                           \ Stuur leds aan
    ;

: SAMPLE-ADC        ( -- u )            \ Elke tiende AD-omzetting op de stack
    0                                   \ Dummy 
    10 0 DO                             \ Toon tien metingen op leds
        DROP                            \ Verwijder oude AD-omzetting
        ADC  DUP BEELD-AF               \ Lees ADC, en toon op LEDS
        1 100MS                         \ Wacht 1/10 sec.
    LOOP
    ;

20 VARIABLES DATA                       \ Ruimte voor 20 metingen

: ZEND-TIEN-METINGEN    ( offset -- )   \ Stuur 10 metingen over
    10 0 DO
        DUP I + FROM DATA .DATA
    LOOP
    DROP
    ;

: DATALOGGER    ( -- )                  \ Een eenkanaal analoge datalogger
    SETUP  RS232-SETUP
    -1 SETDIR LEDS                      \ Zet I/O richting
    1 SETDIR ADC-KLOK
    0 SETDIR ADC-DATA  SET ADC-DATA     \ Ingang met pullup
    1 SETDIR ADC-SELECTEER
    DECIMAL  PAGE                       \ Scherm schoon
    ." ADC naar RS232 data logger"      \ Opstart melding
    KNIPPER 
    ADC DROP                            \ Eerste AD-omzetting is ongeldig
    BEGIN
        CR  20 0 DO                     \ Bewaar 20 samples
            SAMPLE-ADC                  \ Sample ADC en toon op LEDS
            I TO DATA                   \ Sla meting I op
        LOOP
        CR 00 ZEND-TIEN-METINGEN        \ Eerste tien
        CR 10 ZEND-TIEN-METINGEN        \ Tweede tien
    AGAIN
    ;  MAIN

\ Einde
