\ Besturing van twee servo's, 300 bytes.

90S2313                                 \ Selecteer target
NEEDS TARGET.FRT                        \ Laad cpu specifiek deel

PORTD SFR UIT
PORTD 4 BIT-SFR SERVO0                  \ Twee servo uitgangen
PORTD 5 BIT-SFR SERVO1
TIMSK   SFR TIMER-INT                   \ Timer interrupt besturing
TCCR0   SFR TIMER-CONTROLE              \ Timer 0 mode
TCNT0   SFR TIMER                       \ Timer 0 werkregister

2 CONSTANT #S                           \ Aantal servos
REGISTER SERVO#                         \ Servo nummer
REGISTER TELLER                         \ Verleng pauze puls
#S VARIABLES SERVOS                     \ #S servostand geheugens

\ Als +n is 0 tot 63 dan is de pulsbreedte 1 tot 2 millisec.
: SERVO             ( +n s -- )         \ Zet nieuwe stand +n van servo s
    >R  62 + INVERT  R> TO SERVOS       \ en converteer naar timer waarde
    ;                                   \ voor servo s

CODE SERVO-EXIT     ( -- )              \ 13 ticks
    ADR TIMER R16 OUT,                  \ Zet pulslengte (time-out tijd)
    ADR SERVO# INC,                     \ Naar volgende servo
    SREG R17 OUT,                       \ Herstel status register
    R17 POP,                            \ en gebruikte registers
    R16 POP,
    RETI,
END-CODE

\ Deze interrupt gebruikt 15.800 cycles van 4.000.000 ~ 0.4 % cpu tijd.
CODE SERVO-INTERRUPT ( -- )             \ Aansturing voor 2 servo's
    R16 PUSH,                           \ Bewaar te gebruiken regsisters
    R17 PUSH,
    R17 SREG IN,                        \ en het status register
    R16 ADR SERVO# MOV,                 \ Welke servo is aan de beurt
    R16 0 CPI,
    ZER IF,                             \ Servo-0
        ADR SERVO0 SBI,                 \ Puls is hoog
        R16 0 SERVOS LDS,               \ Haal pulslengte op
        ' SERVO-EXIT GJMP,              \ 15+13=28
    THEN,
    R16 1 CPI,
    ZER IF,                             \ Servo-1
        ADR SERVO0 CBI,
        ADR SERVO1 SBI,
        R16 4 LDI,                      \ Zet puls strek teller
        ADR TELLER R16 MOV,
        R16 1 SERVOS LDS,
        ' SERVO-EXIT GJMP,              \ 20+13=33
    THEN,
    ADR SERVO1 CBI,
    ADR TELLER DEC,                     \ Herhaal 3 maal
    ZER IF,
        R16 -1 LDI,                     \ Begin weer bij het begin
        ADR SERVO# R16 MOV,
    THEN,
    R16 CLR,
    ' SERVO-EXIT GJMP,                  \ 32+13=45
END-CODE  T0-OVERFLOW

: SETUP-SERVO       ( -- )
    CLEAR SERVO#                        \ Begin bij servo-0
    2 TO TIMER-INT                      \ Timer-0 interrupt aan
    3 TO TIMER-CONTROLE                 \ Prescaler= (deel cpu klok door) 64
    SET TIMER                           \ Zet iets in het timer-0 register
    $30 SETDIR UIT                      \ Bit 4 en 5 zijn uitgangen
    INTERRUPT-ON                        \ Activeer interrupt mechanisme
    ;

\ Demonstratie programma met twee servo's

: 100MS         ( u -- )                \ Wacht u maal 100 millisec.
    FOR  100 MS  NEXT
    ;

: BEWEEG-SERVOS ( -- )
    SETUP  SETUP-SERVO                  \ Initialiseer stacks en servo hardware
    BEGIN                               \ Start servo demo
        25 0 DO
            I 3 *  0 SERVO              \ Zet servo 0, dan
            72 I 3 * -  1 SERVO 3 100MS \ servo 1 en wacht even
        LOOP
        20 100MS
    AGAIN
    ;  MAIN
