\ ----------------------------------------------------------------------
\ LANGUAGE    : AVR ByteForth vsn 2.00
\ PROJECT     : AVR-EEP.FRT
\ DESCRIPTION : Gebruik van de interne AVR EEPROM
\ CATEGORY    : Voorbeeld, afmeting: 204 bytes.
\ AUTHOR      : Willem Ouwerkerk, March 05, 2000
\ LAST CHANGE : Willem Ouwerkerk, March 05, 2000
\ ----------------------------------------------------------------------

90S2313                                 \ Selecteer target
NEEDS TARGET.FRT                        \ Laad cpu specifiek deel

EEAR SFR ADRES                          \ EEPROM adres register
EEDR SFR DATA                           \ EEPROM data register
EECR 0 BIT-SFR LEES                     \ EEPROM lees opdracht
EECR 1 BIT-SFR SCHRIJF                  \ EEPROM schrijf opdracht
EECR 2 BIT-SFR BEVEILIG                 \ EEPROM schrijf beveiliging opdracht

: EE@           ( a -- x )
    TO ADRES                            \ Zet adres reg.
    SET LEES                            \ Lees data op dat adres
    BEGIN  LEES 0= UNTIL                \ Wacht tot data gelezen is
    DATA                                \ Data op stack
    ;

: EE!           ( x a -- )
    BEGIN  SCHRIJF 0= UNTIL             \ Vorige schrijf opdr. afgerond
    TO ADRES  TO DATA                   \ Ja, zet adres en data klaar
    SET BEVEILIG  SET SCHRIJF           \ geef schrijf opdracht
    ;

\ Een demonstratie van het gebruik van het EEPROM

PORTB SFR LEDS                          \ 8 ledjes op poort B
PORTD SFR INGANG                        \ 8 schakelaars op poort D
PORTD 0 BIT-SFR BEWAAR                  \ Eerste schak. activeert schrijven

: SEC           ( u -- )                \ Wacht u sec.
    FOR  4 FOR  250 MS  NEXT  NEXT
    ;

: DEMO          ( -- )
    SETUP  -1 SETDIR LEDS               \ Leds zijn uitgangen
    0 SETDIR INGANG                     \ Schakelaars zijn ingangen
    CLEAR LEDS  1 SEC                   \ Leds aan, even wachten
    BEGIN
        SET LEDS  1 SEC                 \ Leds uit, even wachten
        FROM BEWAAR 0= IF               \ PD0 ingedrukt ?
            CLEAR LEDS  1 SEC           \ Ja, leds even aan
            20 FOR                      \ Sample 20 monsters
                FROM INGANG             \ Lees schakelaars
                DUP TO LEDS             \ Toon op leds
                I' EE!  1 SEC           \ Sla op in EEPROM en wacht
            NEXT
        THEN
        20 FOR                          \ Toon EEPROM data op leds
            I' EE@ TO LEDS  250 MS      \ en wacht wat minder lang
        NEXT
    AGAIN
    ; MAIN

                              ( Einde )
