\ ==============================================================
\ [TITLE]     AVR ByteForth Hitachi LCD example. 
\ [FILE]      LCD.FRT
\ [CATEGORY]  Example, length: 436 bytes.
\ [VSN]       1.00
\ [CREATED]   June 27, 1995, Willem Ouwerkerk 
\ [LASTCHNGD] June 27, 2000, Willem Ouwerkerk 
\ [COPYRIGHT] De Elektrise Schaar (C) 2000-
\ [USES]      AVR ByteForth cross compiler
\ [PURPOSE]   An Character LC-display on STK200 kit
\             Use a display with the Hitachi HD44780A00H chip,
\             and 8 bits LCD I/O communication (4MHz). 
\ ==============================================================

90S8515                                 \ Target CPU
NEEDS TARGET.FRT

ATOM INLINE$        ATOM MS             \ Import these macros

\ This method is not valid for a four line UC-16401 display.
\ CR and AT-XY  does not work properly. This because the
\ display has only two lines internally. These lines are
\ interlaced on the four display lines. 

\ LCD-display dimensions and cursor position

16 CONSTANT C/L                         \ Screen width
FLAG DATA?                              \ Lcd data type, true is CHAR

PORTA SFR LCD                           \ I/O port for LCD using 6 bits
PORTC 6 BIT-SFR RS                      \ Select character or instruction
PORTC 7 BIT-SFR ENABLE                  \ Pulse high when data is ready
PORTD 6 BIT-SFR R/W                     \ Read from LCD=high, write-low

\ When DATA? is true, byte is accepted as data. 
\ When DATA? is false, byte is accepted as instruction.

\ Zet een karakter char op het LC-display met 1 regel.
CODE EMIT       ( char -- )             \ Lcd
    ADR RS SBI,                         \ Bit PC.6 is RS=1 is data
    R16 X+ LD,
    ADR LCD R16 OUT,
    ADR ENABLE SBI,                     \ Bit PC.7 is E
    ADR ENABLE CBI,
    R16 50 LDI,                         \ Wait about 40 usec.
    DO, R16 LOOP,
    RET,
END-CODE

\ Stuur instructie x naar een LC-display.
CODE INSTR      ( x -- )                \ Lcd
    ADR RS CBI,
    ' EMIT 1. D+ RJMP,
END-CODE

\ Maak het LC-display schoon en zet de cursor in linker bovenhoek.
: PAGE      ( -- )                  \ Lcd Clean LCD and restore cursor
    1 INSTR  2 MS
    ;

\ Initialiseer een LC-display. Zet 4 bits data bus, maak het 
\ display schoon, zet de cursor uit en zet de display richting 
\ van links naar rechts.
: INIT-LCD          ( -- )              \ Lcd
    $FF SETDIR LCD                      \ All bits outputs
    -1 SETDIR RS                        \ Register select is output
    -1 SETDIR ENABLE                    \ Enable is output
    -1 SETDIR R/W  CLEAR R/W            \ R/W is output and always low
    16 MS                               \ Wait and set 8 bits interface mode
    3 FOR  $30 INSTR 5 MS  NEXT
    $30 INSTR                           \ Eight bits interf. one line display
    $08 INSTR                           \ Display off
    PAGE                                \ Clear display
    $06 INSTR                           \ Shift right
    $0C INSTR                           \ Display on, cursor off
    ;

\ Zet u karakters vanaf het ROM adres $addr naar het LC-display.
: TYPE          ( $addr u -- )          \ Lcd Output u chars from $addr
    ?DUP IF
        FOR  COUNT EMIT  NEXT  
    THEN
    2DROP
    ;

PORTB SFR LEDS

PORTD 0 BIT-SFR IN1
PORTD 1 BIT-SFR IN2

REGISTER TELLER
REGISTER SCHUIF

: SCHUIF-OOK    ( -- )
    FROM IN2 0= IF
        SCHUIF INVERT TO LEDS  50 MS
        SCHUIF 2*  DUP 0= IF  1+  THEN  TO SCHUIF
    ELSE
        50 MS
    THEN
    ;

: WACHT         ( -- )
    FROM IN1 0= IF
        5 FOR
            TELLER INVERT TO LEDS  10 MS
            INCR TELLER
        NEXT
    ELSE
        SCHUIF-OOK
    THEN
    ;

22. ALLOT

: DEMO           ( -- )                  \ Display character set on LCD
    SETUP  INIT-LCD  
    -1 SETDIR LEDS  0 SETDIR IN1  0 SETDIR IN2
    CLEAR LEDS  ." Hallo Wim. "  50 MS  SET LEDS  
    CLEAR TELLER  1 TO SCHUIF  250 MS  0
    BEGIN
        5 FOR  WACHT  NEXT  PAGE
        C/L FOR
            DUP EMIT  1+  WACHT
        NEXT
    AGAIN
    ;  MAIN

\ End
