\ ----------------------------------------------------------------------
\ LANGUAGE    : AVR ByteForth vsn 2.00
\ PROJECT     : High level interrupts
\ DESCRIPTION : ByteForth uitbreiding met colon-interrupts
\ CATEGORY    : Applicatie, afmeting: 208 bytes.
\ AUTHOR      : Willem Ouwerkerk, March 26, 2002
\ LAST CHANGE : Willem Ouwerkerk, April 1, 2002
\ ----------------------------------------------------------------------

90S2313                                 \ Selecteer target
NEEDS TARGET.FRT                        \ Laad cpu specifiek deel


\ Een voorbeeld van een hi-level interrupt

TCCR0 SFR TIMER-CONTROLE0               \ Hardware registers
TCNT0 SFR COUNTER0
TIMSK SFR TIMER-INT

PORTB SFR LEDS

VARIABLE TELLER

: SETUP-TELLER  ( -- )
    -1 SETDIR LEDS          ( Leds zijn uitgang )
    0 TO COUNTER0           ( Initialiseer timer-0 register )
    2 TO TIMER-INT          ( Sta timer-0 interrupts toe )
    5 TO TIMER-CONTROLE0    ( Zet timer-0 prescaler op XT/1024 )
    INTERRUPT-ON            ( Activeer interrupt mechanisme )
    ;


DOC umentatie over hi-level interrupts

Hieronder een simpel voorbeeld van een hi-level interrupt. 
Er is een teller aangebracht in een timerinterrupt.
De teller wordt uitgelezen in de hoofdroutine en op 
8 leds getoond. Belangrijk voor high-level interrupts
is dat de interrupt routine 'stack neutraal' is !!
Let er ook op dat de hardwarestack voldoende diep is !

De optimizer wordt expres even uitgezet zodat de routine
ook echt de Forth stack gebruikt.

ENDDOC


OPTIMIZER-OFF

: VOORBEELD     ( -- )
    PUSHALL             ( Save Forth werkgebied )
    0 TO COUNTER0       ( timer-0 start weer op nul )
    FROM TELLER 1 +     ( Verhoog de teller variabele )
    TO TELLER
    POPALL              ( Herstel Forth werkgebied )
    ;INT  T0-OVERFLOW   ( Zet timer-0 interrupt vector )

OPTIMIZER-ON

:MAIN           ( -- )
    SETUP-TELLER
    CLEAR LEDS 250 MS  SET LEDS 250 MS
    BEGIN
        FROM TELLER INVERT TO LEDS
    AGAIN
    ;

                              ( Einde )
