\ ----------------------------------------------------------------------
\ LANGUAGE    : AVR ByteForth vsn 2.00
\ PROJECT     : High level interrupts
\ DESCRIPTION : ByteForth uitbreiding met colon-interrupts
\ CATEGORY    : Applicatie, afmeting: 128 bytes.
\ AUTHOR      : Willem Ouwerkerk, March 26, 2002
\ LAST CHANGE : Willem Ouwerkerk, April 2, 2002
\ ----------------------------------------------------------------------

90S2313                                 \ Selecteer target
NEEDS TARGET.FRT                        \ Laad cpu specifiek deel


\ Een voorbeeld van een hi-level interrupt

TCCR0 SFR TIMER-CONTROLE0               \ Hardware registers
TCNT0 SFR COUNTER0
TIMSK SFR TIMER-INT

PORTB SFR LEDS

VARIABLE TELLER

: SETUP-TELLER  ( -- )
    -1 SETDIR LEDS          ( Leds zijn uitgang )
    0 TO COUNTER0           ( Initialiseer timer-0 register )
    2 TO TIMER-INT          ( Sta timer-0 interrupts toe )
    5 TO TIMER-CONTROLE0    ( Zet timer-0 prescaler op XT/1024 )
    INTERRUPT-ON            ( Activeer interrupt mechanisme )
    ;


DOC umentatie over high-level interrupts

Hieronder een simpel voorbeeld van een hi-level interrupt. 
Er is een teller aangebracht in een timerinterrupt.
De teller wordt uitgelezen in de hoofdroutine en op 
8 leds getoond. Belangrijk voor high-level interrupts
is dat de interrupt routine 'stack neutraal' is !!
Let er ook op dat de hardwarestack voldoende diep is !
Deze versie gebruikt de PUSH & POP prefixen om handmatig
de gebruikte registers te bewaren. Het kan dus toch
helemaal in high-level (Byte)Forth.

ENDDOC


16    (REGISTER) ACCU   ( ByteForth's belangrijkste werkregister )
SREG  SFR STATUS-REG    ( AVR Status register )

: VOORBEELD     ( -- )
    PUSH ACCU           ( Save Forth werkgebied )
    PUSH STATUS-REG

    0 TO COUNTER0       ( timer-0 start weer op nul )
    INCR TELLER         ( Verhoog de teller variabele )

    POP STATUS-REG      ( Herstel Forth werkgebied )
    POP ACCU     
    ;INT  T0-OVERFLOW   ( Zet timer-0 interrupt vector )


:MAIN           ( -- )
    SETUP-TELLER
    CLEAR LEDS 250 MS  SET LEDS 250 MS
    BEGIN
        FROM TELLER INVERT TO LEDS
    AGAIN
    ;

                              ( Einde )
