\ ==============================================================
\ [TITLE]     AVR ByteForth Hitachi LCD example. 
\ [FILE]      LCD.FRT
\ [CATEGORY]  Example, length: 316 bytes.
\ [VSN]       1.00
\ [CREATED]   June 27, 1995, Willem Ouwerkerk 
\ [LASTCHNGD] August 24, 2000, Willem Ouwerkerk 
\ [COPYRIGHT] De Elektrise Schaar (C) 2000-
\ [USES]      AVR ByteForth cross compiler
\ [PURPOSE]   Put character LC-display on STK200 kit.
\             Use a display with the Hitachi HD44780A00H chip,
\             and 8 bits LCD I/O communication (4MHz). 
\ ==============================================================

90S4414                                 \ Target CPU
NEEDS TARGET.FRT

ATOM INLINE$        ATOM MS             \ Import these macros

\ This method is not valid for a four line UC-16401 display.
\ CR and AT-XY  does not work properly. This because the
\ display has only two lines internally. These lines are
\ interlaced on the four display lines. 

\ LCD-display dimensions and cursor position

16 CONSTANT C/L                         \ Screen width

PORTA SFR LCD                           \ I/O port for LCD using 8 bits
PORTC 6 BIT-SFR RS                      \ Select character or instruction
PORTC 7 BIT-SFR ENABLE                  \ Pulse high when data is ready
PORTD 6 BIT-SFR R/W                     \ Read from LCD=high, write-low

\ Zet een karakter char op het LC-display met 1 regel.
CODE EMIT       ( char -- )             \ Lcd
    ADR RS SBI,                         \ Bit PC.6 is RS=1 is data
    R16 X+ LD,
    ADR LCD R16 OUT,
    ADR ENABLE SBI,                     \ Bit PC.7 is E
    ADR ENABLE CBI,
    R16 50 LDI,                         \ Wait about 40 usec.
    DO, R16 LOOP,
    RET,
END-CODE

\ Stuur instructie x naar een LC-display.
CODE INSTR      ( x -- )                \ Lcd
    ADR RS CBI,
    ' EMIT 1. D+ RJMP,
END-CODE

\ Maak het LC-display schoon en zet de cursor in linker bovenhoek.
: PAGE          ( -- )                  \ Lcd Clean LCD and restore cursor
    1 INSTR  2 MS
    ;

\ Initialiseer een LC-display. Zet 4 bits data bus, maak het 
\ display schoon, zet de cursor uit en zet de display richting 
\ van links naar rechts.
: INIT-LCD          ( -- )              \ Lcd
    $FF SETDIR LCD                      \ All bits outputs
    -1 SETDIR RS                        \ Register select is output
    -1 SETDIR ENABLE                    \ Enable is output
    -1 SETDIR R/W  CLEAR R/W            \ R/W is output and always low
    16 MS                               \ Wait and set 8 bits interface mode
    3 FOR  $30 INSTR 5 MS  NEXT
    $30 INSTR                           \ Eight bits interf. one line display
    $08 INSTR                           \ Display off
    PAGE                                \ Clear display
    $06 INSTR                           \ Shift right
    $0C INSTR                           \ Display on, cursor off
    ;

\ Zet u karakters vanaf het ROM adres $addr naar het LC-display.
: TYPE          ( $addr u -- )          \ Lcd Output u chars from $addr
    ?DUP IF
        FOR  COUNT EMIT  NEXT
    THEN
    2DROP
    ;

: DEMO          ( -- )                  \ Display character set on LCD
    SETUP  INIT-LCD  ." Wim's STK200 Lcd"
    10 FOR  100 MS  NEXT  -1
    BEGIN
        250 MS  PAGE
        C/L FOR
            1+ DUP EMIT  50 MS
        NEXT
    AGAIN
    ;  MAIN

\ End
