\ -----------------------------------------------------------
\ PROJECT     : I2C thermometer with LM75CIM, 236 bytes
\ AUTEUR      : Willem Ouwerkerk, Februari 29, 2000
\ This example runs on an STK200 board fitted with a AT90S2313
\ -----------------------------------------------------------

90S2313                                 \ Select target
1 0 0 MEMORY                            \ Adjust memory needs
NEEDS TARGET.FRT                        \ Load target constants
4 SET-CRYSTAL                           \ Set used Xtal

\ Routines copied from library file RS232.FRT
CODE RS232-SETUP                        \ RS232
    R16 25 LDI,                         \ Select 9600 baud
    UBRR R16 OUT,
    R16 $18 LDI,
    UCR R16 OUT,                        \ Enable receiver & transmitter
    RET,                                \ Reset interrupt, set 8 bit data.
END-CODE

CODE EMIT           ( char -- )         \ RS232 Char to RS232
    R16 X+ LD,
    UDR R16 OUT,
    BEGIN,  
        USR 6 SBIS,
    AGAIN,
    RET,
END-CODE

\ General I2C primitives copied from library I2CPRIMC.FRT
PORTD 4  BIT-SFR SDA                    \ I2C data line control
DDRD 4 BIT-SFR SDAD
PIND 4 BIT-SFR SDAIN
PORTD 5  BIT-SFR SCL                    \ I2C clock line control
DDRD 5 BIT-SFR SCLD

CODE WACHT      ( -- )                  \ Wacht 4.75 us ( 8/14 bytes )
    R18 3 LDI,                          \ .25
    DO, R18 LOOP,                       \ 3 * .75 + .5
    RET,                                \ 1.75
END-CODE

: START-BIT     ( -- )                  \ Generate I2C-start
    SET SDA  SET SCL  
    CLEAR SDA  WACHT
    CLEAR SCL  WACHT
    ;

: STOP-BIT      ( -- )                  \ Generate I2C stopbit
    CLEAR SDA  SET SCL  WACHT  SET SDA
    ;

: NACK-BIT      ( -- )                  \ Generate no-ackbit
    SET SDA  SET SCL  WACHT  CLEAR SCL  WACHT
    ;

CODE BYTE-UIT   ( x -- )                \ 44 bytes code, high level ?? bytes
    R16 X+ LD,
    R17 8 LDI,
    DO,
        R16 7 SBRC,                     \ Data bit 7 high ?
        ADR SDA SBI,                    \ Yes, dataline high
        R16 7 SBRS,                     \ No, data bit 7 low ?
        ADR SDA CBI,                    \ Yes, dataline low
        R16 LSL,                        \ Get next databit
        ADR SCL SBI,                    \ Generate clockpulse
        ' WACHT RCALL,
        ADR SCL CBI,
        ' WACHT RCALL,
    R17 LOOP,
    ADR SDA SBI,                        \ Test ack bit from slave
    ADR SCL SBI,
    ADR SDAD CBI,                       \ Dataline is input !
    ' WACHT RCALL,
    ADR SDAD SBI,
    ADR SDAIN SBIC,                     \ NoAck received ?
    ' STOP-BIT RJMP,                    \ Yes, generate stopcondition
    ADR SCL CBI,                        \ No, finish clockpulse
    ' WACHT RJMP,
END-CODE

CODE BYTE-IN    ( -- x )                \ 26 bytes code, high level 44 bytes
    R17 8 LDI,
    ADR SDAD CBI,
    DO,
        ADR SCL SBI,
        ' WACHT RCALL,
        CLC,
        ADR SDAIN SBIC,
        SEC,
        R16 ROL,
        ADR SCL CBI,
        ' WACHT RCALL,
    R17 LOOP,
    ADR SDAD SBI,
    -X R16 ST,
    RET,
END-CODE

$91 CONSTANT LM75-ID                    \ LM75 base address

: TEMPERATUUR   ( -- u )                \ Read temperature as
    START-BIT                           \ an 8 bit value
    LM75-ID  BYTE-UIT
    BYTE-IN  NACK-BIT                   \ msb
    STOP-BIT 
    ;

\ Define high level control for LM75

PORTB SFR LEDS                          \ Use leds on port B

: SECONDEN      ( u -- )                \ wait u seconds
    FOR  4 FOR  250 MS  NEXT  NEXT
    ;

:MAIN           ( -- )
    RS232-SETUP                         \ Initialise RS232 at 9600 baud
    1 SETDIR SDA  1 SETDIR SCL          \ Initialise I2C
    -1 SETDIR LEDS  CLEAR LEDS          \ All leds outputs
    BEGIN
            1 SECONDEN                  \ Wait a moment
            TEMPERATUUR  DUP EMIT       \ Read LM75, send data binairy
            INVERT TO LEDS              \ to RS232 & show data on leds
    AGAIN ;

                               ( Einde )