\ ----------------------------------------------------------------------
\ LANGUAGE    : AVR ByteForth
\ PROJECT     : Demonstratie software
\ DESCRIPTION : 'Real Time Clock' op port B
\ CATEGORY    : Voorbeeld, afm: 336 bytes.
\ AUTHOR      : Willem Ouwerkerk, Mei 23, 2000
\ LAST CHANGE : Willem Ouwerkerk, Mei 26, 2000
\ ----------------------------------------------------------------------

mega103                                 \ Compileer voor ATmega103
NEEDS TARGET.FRT                        \ Laad labels & vectoren voor AVR

PORTB   SFR LEDS                        \ Poort B met 8 leds
ASSR    SFR T0-STATUS                   \ Timer 0 besturing
TCNT0   SFR T0-TELLER
OCR0    SFR T0-VERGELIJK
TCCR0   SFR T0-CONTROLE
TIMSK   SFR TIMER-MASK

REGISTER SEC                            \ Sec. teller
REGISTER MIN
REGISTER UUR

: SETUP-KLOK    ( -- )                  \ PB.4 met sec. tik
    8 TO T0-STATUS                      \ Gebruik extern kristal
    CLEAR T0-TELLER                     \ Begin te tellen vanaf nul
    32 TO T0-VERGELIJK                  \ Teller loopt af na 32 tikken
    $0F TO T0-CONTROLE                  \ PB.5 is uitgang en deler = 1024
    -1 SETDIR LEDS                      \ Leds zitten op uitgang
    2 TO TIMER-MASK                     \ Sta timer-0 vergelijk intrpt toe
    CLEAR SEC  CLEAR MIN  CLEAR UUR     \ Begin op nul
    INTERRUPT-ON                        \ Interrupt mechanisme aan
    ;

CODE TEL        ( -- )                  \ Tel seconden
    R16 PUSH,
    R16 SREG IN,
    R16 PUSH,
    ADR SEC INC,
    R16 POP,
    SREG R16 OUT,
    R16 POP,
    RETI,
END-CODE  T0-COMPARE    

: 10MS          ( u - )                 \ Wacht u maal 10 millisec.
    ?DUP IF  0 DO  10 MS  LOOP  THEN
    ;

: KNIPPER       ( -- )                  \ Visualiseer opstarten
    CLEAR LEDS      50 10MS             \ Led aan
    SET LEDS        50 10MS             \ Led uit
    ;

: TIKTAK        ( -- )                  \ Maak 24 uurs klok
    SEC 60 = IF  
        CLEAR SEC  INCR MIN
        MIN 60 = IF
            CLEAR MIN  INCR UUR
            UUR 24 = IF
                CLEAR UUR
            THEN
        THEN
    THEN
    ;

: TOON          ( -- )                  \ Toon klok op leds
    0  SEC 1 AND IF  128 OR  THEN
    SEC 30 < IF  
        MIN
    ELSE
        UUR
    THEN
    OR INVERT TO LEDS
    ;

: KLOK          ( -- )
    SETUP  SETUP-KLOK  KNIPPER          \ Initialiseer
    14 TO UUR  26 TO MIN
    BEGIN  TIKTAK  TOON  AGAIN          \ Toon teller
    ;  MAIN                             \ KLOK start op na een reset

                               ( End )
