\ ==============================================================
\ [TITLE]     AVR ByteForth Round Robin Multitasking
\ [FILE]      TASKING0.FRT
\ [CATEGORY]  Multitasker voorbeeld, lengte: 1282 bytes.
\ [VSN]       1.10
\ [CREATED]   Mei 19, 2002, Willem Ouwerkerk 
\ [LASTCHNGD] Mei 27, 2002, Willem Ouwerkerk 
\ [COPYRIGHT] De Elektrise Schaar (C) 2002-
\ [USES]      AVR ByteForth cross compiler
\ [PURPOSE]   Multitasking op AVR met I/O naar een LCD op een
\             STK200 kit, tevens RS232 uitvoer, Bit Invoer en
\             bit uitvoer naar 8 leds. Als vier afzonderlijke
\             taken. Meerdere malen MS via een timer interrupt.
\ ==============================================================

mega8515                                \ Target CPU
NEEDS TARGET.FRT
ATOM INLINE$                            \ Importeer macros
INCLUDE ROBIN6.FRT                      \ Multitasker met vier taken
INCLUDE RS232.FRT                       \ Taak 3: RS232


\ LCD-display dimensions and cursor position

16 CONSTANT C/L                         \ Screen width
FLAG DATA?                              \ Lcd data type, true is CHAR

PORTA SFR LCD                           \ I/O port for LCD using 6 bits
PORTC 6 BIT-SFR RS                      \ Select character or instruction
PORTC 7 BIT-SFR ENABLE                  \ Pulse high when data is ready
PORTD 6 BIT-SFR R/W                     \ Read from LCD=high, write-low

\ When DATA? is true, byte is accepted as data. 
\ When DATA? is false, byte is accepted as instruction.

\ Zet een karakter char op het LC-display met 1 regel.
CODE EMIT       ( char -- )             \ Lcd
    ADR RS SBI,                         \ Bit PC.6 is RS=1 is data
    R16 X+ LD,
    ADR LCD R16 OUT,
    ADR ENABLE SBI,                     \ Bit PC.7 is E
    ADR ENABLE CBI,
    R16 50 LDI,                         \ Wait about 40 usec.
    DO, R16 LOOP,
    RET,
END-CODE

\ Stuur instructie x naar een LC-display.
CODE INSTR      ( x -- )                \ Lcd
    ADR RS CBI,
    ' EMIT 1. D+ RJMP,
END-CODE

\ Maak het LC-display schoon en zet de cursor in linker bovenhoek.
: PAGE      ( -- )                  \ Lcd Clean LCD and restore cursor
    1 INSTR  2 MS1
    ;

\ Initialiseer een LC-display. Zet 4 bits data bus, maak het 
\ display schoon, zet de cursor uit en zet de display richting 
\ van links naar rechts.
: INIT-LCD          ( -- )              \ Lcd
    $FF SETDIR LCD                      \ All bits outputs
    -1 SETDIR RS                        \ Register select is output
    -1 SETDIR ENABLE                    \ Enable is output
    -1 SETDIR R/W  CLEAR R/W            \ R/W is output and always low
    16 MS1                              \ Wait and set 8 bits interface mode
    3 FOR  $30 INSTR 5 MS1  NEXT
    $30 INSTR                           \ Eight bits interf. one line display
    $08 INSTR                           \ Display off
    $01 INSTR 2 MS1                     \ Clear display
    $06 INSTR                           \ Shift right
    $0C INSTR                           \ Display on, cursor off
    ;

\ Zet u karakters vanaf het ROM adres $addr naar het LC-display.
: TYPE          ( $addr u -- )          \ Lcd Output u chars from $addr
    ?DUP IF
        FOR  COUNT EMIT  PAUSE  NEXT  
    THEN
    2DROP
    ;

: RTYPE         ( addr u -- )
    FOR  @+ EMIT  PAUSE  NEXT  2DROP
    ;

: SPACE  BL EMIT ;

NEEDS NUMBERS.FRT                       \ Laad ook getal I/O


PORTB SFR LEDS                          \ Uitvoer
PORTD SFR TOETS                         \ Invoer
VARIABLE TIJD

: LCD-DEMO  ( n1 -- n2 )                \ Taak 0: Bestuur LCD (hoofdtaak)
    SCHAKELAAR IF                       \ Bit geheugen bepaald display actie
        250 MS1  PAGE
        C/L FOR
            DUP 128 32 within IF DROP 32 THEN
            DUP EMIT  1+  100 MS1
        NEXT
    ELSE
        250 MS1  PAGE ." Ik tel "
        DUP U.  1+
    THEN
    ;

: SCHUIF    ( -- )                      \ Taak 1: Maak looplicht patroon
    0
    BEGIN
        2*  DUP 0= IF  DROP 1  THEN
        DUP INVERT TO LEDS 
        FROM TIJD MS2
    AGAIN
    ;

: LEES      ( -- )                      \ Taak 2: Lees toets data in
    BEGIN
        FROM TOETS INVERT  $3C AND 
        1 UMAX 2* 2* TO TIJD  PAUSE
    AGAIN
    ;

\ De hoofdtaak start de drie achtergrond taken op !!
: DEMO           ( -- )
    SETUP  COMPARE-AAN  RS232-SETUP \ RS232 op 9600 baud, het multitasker
    CLEAR TASK#  CLEAR LEDS         \ mechanisme, de leds, schakelaars en
    -1 SETDIR LEDS  66 SETDIR TOETS \ de timer-interrupt.
    ['] SCHUIF  1 ACTIVATE          \ Zet achtergrondtaken klaar
    ['] LEES    2 ACTIVATE
    ['] RS232   3 ACTIVATE
    INIT-LCD  ." Round Robin Task"  \ Setup LCD en start LCD hoofdtaak op
    4 FOR  250 MS1  NEXT   BL       \ Wacht en begin demo met spatie
    DECIMAL  SET SCHAKELAAR
    BEGIN  LCD-DEMO  AGAIN
    ;  MAIN

\ Eind
