\ -------------------------------------------------------
\ LANGUAGE    : AVR ByteForth vsn 2.00
\ DESCRIPTION : Kleine ultrasoon afstandmeter, 196 Bytes
\ AUTHOR      : Willem Ouwerkerk, Oktober 7, 2002
\ -------------------------------------------------------
\ Deze interface werkt ook met code die een echte GP2D02 uitleest.
\ -------------------------------------------------------

90S2313                         \ Selecteer target
1 0 0 MEMORY                    \ Strip geheugengebruik
4 SET-CRYSTAL                   \ Gebruik 4 MHz type
NEEDS TARGET.FRT                \ Laad cpu specifiek deel

TCCR1B SFR T1CONTROL            \ Toegang tot hardware
TCNT1L SFR TIMER1L
TCNT1H SFR TIMER1H
TIFR   SFR TIMERVLAGGEN
PORTB  SFR I/O
PORTD  SFR DRIVER

\ Zender macro's, hoog puls, laag puls en ruststand.
MACRO: HOOG   $24 TO DRIVER ;
MACRO: LAAG   $18 TO DRIVER ;
MACRO: RUST   $1C TO DRIVER ;

\ Wacht 12,25 micosec.
: WACHT             ( -- )      12 FOR NEXT ;

\ Uitgang van comparator (reflectie detector).
ACSR 5 BIT-SFR REFLECTIE?

\ Onthoud het meetresultaat (inhoud timer-0).
REGISTER RESULTAAT

\ Konstanten die de maximale meetafstand bepalen er wordt
\ uitgegaan van een klok van 4 MHz. De geluidssnelheid is
\ een konstante van 0.0341 cm per microseconde.

\ 1000 x 16us x .0341cm /2 = ~272 cm, max. meettijd = 16.00 ms
\  920 x 16us x .0341cm /2 = ~251 cm, max. meettijd = 14.72 ms
\  760 x 16us x .0341cm /2 = ~207 cm, max. meettijd = 12.16 ms
\  560 x 16us x .0341cm /2 = ~153 cm, max. meettijd =  8.96 ms
\  400 x 16us x .0341cm /2 = ~109 cm, max. meettijd =  6.40 ms
 -760. 2CONSTANT US-LIMIET

\ Afstand limiet voor het rekenen in de software.
US-LIMIET D2/ D2/  DROP CONSTANT LIMIET

\ Timer-0 met een prescaler=8 = xt/64 (1 tik = 16us bij 4 MHz).
MACRO: SETUP-US     ( -- )      3 TO T1CONTROL ;

\ Zend 40 KHz pulstreintje.
MACRO: GOLFTREIN    ( -- )      4 FOR  HOOG WACHT  LAAG WACHT  NEXT  RUST ;

\ Zet timer-0 klaar en start hem.
: START-METING      ( -- )
    $80 TO TIMERVLAGGEN         \ Reset timer-1 vlag
    US-LIMIET NIP TO TIMER1H    \ Zet timeout tijd
    US-LIMIET DROP TO TIMER1L
    GOLFTREIN ;

\ Wacht op timeout of reflectie en bewaar meetresultaat.
: REFLECTIE         ( -- )
    BEGIN
        REFLECTIE? IF
            TIMER1L TIMER1H     \ Bewaar inhoud
            D2/ D2/ DROP 
            TO RESULTAAT
            EXIT
        THEN
    TIMERVLAGGEN $80 AND UNTIL  \ Timeout timer=0
    CLEAR RESULTAAT ;

\ De echte afstand in RESULTAAT = RESULTAAT * 1.0912 cm.
: UITKOMST          ( -- u )      
    LIMIET RESULTAAT -
    ;

\ Implementeer GP2D02 stijl interface.
\ Deze interface werkt ook met code die een echte GP2D02 uitleest.

PORTD 0 BIT-SFR KLOK            ( Rx )
PORTD 1 BIT-SFR DATALIJN        ( Tx )

\ Wacht hoge klokpuls.
MACRO: KLOKHOOG     ( -- )
    BEGIN  FROM KLOK UNTIL
    ;

\ Wacht op lage klokpuls, reset cpu bij storing.
MACRO: KLOKLAAG     ( -- )
    BEGIN  FROM KLOK 0= UNTIL
    ;

\ Stuur antwoord via GP2D02 achtige interface.
: ANTWOORD          ( u -- )
    KLOKHOOG
    8 FOR
        KLOKLAAG
        DUP 128 AND TO DATALIJN
        KLOKHOOG  2*
    NEXT
    SET DATALIJN  DROP
    ;


\ Toon opstarten en zend na een startpuls een US pulstrein van 
\ 4 perioden. Het resultaat wordt via een GP2D02 interface
\ afgegeven. De maximale meettijd is nu +- 1 + 13 = 14 millisec.
:MAIN               ( -- )
    $FC SETDIR I/O
    $FE SETDIR DRIVER
    SETUP-US
    BEGIN
        KLOKLAAG START-METING REFLECTIE
        UITKOMST  ANTWOORD  1 /MS
    AGAIN ;
