\ ----------------------------------------------------------------------
\ LANGUAGE    : AVR ByteForth vsn 2.00
\ PROJECT     : PBM met timer 0
\ DESCRIPTION : Puls breedte modulator met timer-0
\ CATEGORY    : Applicatie, afmeting: 240 bytes.
\ AUTHOR      : Willem Ouwerkerk, April 8, 2002
\ LAST CHANGE : Willem Ouwerkerk, Juli 22, 2002
\ ----------------------------------------------------------------------

90S2313                                 \ Selecteer target
NEEDS TARGET.FRT                        \ Laad cpu specifiek deel

PORTB 0 BIT-SFR UITGANG                 \ PB.0
TCCR0 SFR TIMER-CONTROLE                \ Hardware registers
TCNT0 SFR TIMER
TIMSK SFR TIMER-INTRPT

#50 CONSTANT CYCLUS                     \ PBM Lengte
REGISTER AANTIJD                        \ PBM aantijd
REGISTER UITTIJD                        \ PBM uittijd

: SETUP-PBM         ( -- )              \ Stel PBM in
    -1 TO TIMER
    2 TO TIMER-INTRPT
    2 TO TIMER-CONTROLE
    1 SETDIR UITGANG
    ;

\ Ongeveer 28 cycles van elke 400 wordt voor PBM gebruikt = 7 %
\ De frequentie is 4.000.000/8 = 500.000/50 = 10 kHz
\ Deze versie regelt weer van 0 % tot 100 % dankzij de aanpassing
\ in SNELHEID daar wordt bij 0 en het maximum de interrupt uitgezet.
CODE PBM            ( -- )              \ PBM opwekking
    ADR UITGANG SBIS,                   \ Uitgang hoog ?
    AHEAD,
        TCNT0 ADR UITTIJD OUT,          \ Ja, zet pulslengte laag
        ADR UITGANG CBI,                \ en de uitgang laag
        RETI,
    THEN,
    TCNT0 ADR AANTIJD OUT,              \ Nee, zet pulslengte hoog
    ADR UITGANG SBI,                    \ en de uitgang hoog
    RETI,
END-CODE  T0-OVERFLOW

: SNELHEID          ( u -- )            \ Zet snelheid van 0 to 50
    ?DUP 0= IF  INTERRUPT-OFF  CLEAR UITGANG  EXIT  THEN
    DUP CYCLUS = IF  INTERRUPT-OFF  DROP  SET UITGANG  EXIT  THEN
    DUP NEGATE TO AANTIJD
    CYCLUS - TO UITTIJD
    INTERRUPT-ON
    ;

: PBM-DEMO          ( -- )
    SETUP  
    CLEAR UITGANG   255 MS
    SET UITGANG     255 MS
    SETUP-PBM
    BEGIN
        CYCLUS FOR
            I' SNELHEID  100 MS
        NEXT
        CYCLUS SNELHEID
        4 FOR  250 MS  NEXT
        0 SNELHEID
        4 FOR  250 MS  NEXT
    AGAIN
    ;  MAIN

                              ( Einde )
