\ ==============================================================
\ [TITLE]     AVR ByteForth seven segment display control
\ [FILE]      7SEGM.FRT
\ [CATEGORY]  Library ( 282 bytes )
\ [VSN]       1.02
\ [CREATED]   June 8, 1998, Willem Ouwerkerk
\ [LASTCHNGD] March 3, 2000, Willem Ouwerkerk
\ [COPYRIGHT] Elektrise Schaar (C) 1996-2000
\ [USES]      AVR ByteForth
\ [PURPOSE]   Control of multiple 7segment displays thru 3 to
\             four wires with use of 4094 shift register.
\ ==============================================================

\G Bevat het aantal aangesloten digits.
4 CONSTANT #DIGITS   ( -- u )           \ 7segm  Aantal gebruikte 7-segm. display's
                                        \ en 74HC4094 chips.

\ u*8 bits uitvoer gebruikt nooit meer dan 3 I/O-bits
PORTB 4 BIT-SFR LED                     \ 7segm  Leds tussen displays
PORTB 5 BIT-SFR KLOK                    \ 7segm  Klok             (13=Clock )
PORTB 6 BIT-SFR DATA                    \ 7segm  Data bitstroom   (14=Data  )
PORTB 7 BIT-SFR KLAAR                   \ 7segm  Data geldig      (15=Strobe)

: SETUP-7SEGM       ( -- )              \ Alle bits zijn uitgangen
    1 SETDIR LED    1 SETDIR KLOK
    1 SETDIR DATA   1 SETDIR KLAAR
    ;

: SCHRIJF-BIT       ( flag -- )         \ 7segm  Schrijf een bit naar uitgang
    TO DATA  CLEAR KLOK  SET KLOK
    ;

: TOON              ( -- )              \ 7segm  Zet data op uitgangen
    SET KLAAR  CLEAR KLAAR
    ;
 
: >DRIVER           ( x -- )            \ 7segm  Schrijf x naar display
    8 FOR                               \ display driver
        DUP 128 AND SCHRIJF-BIT  2*
    NEXT
    DROP
    ;

\ Maak cijfers voor een zevensegment display via de 74HC4094 0 t/m F
\ De digits staan in omgekeerde volgorde op de stack bij CONSTANTS !!!
\ Layout van 7segm. Display: +-A-+
\                            F   B
\ Aansluiting 74HC4094       +-G-+
\  PBCAFDGE                  E   C
\ Q87654321                  +-D-+                
\ --------------------------------------------- \ Opbouw van de tekens
%01110001   %01111001   %01011110   %00111001   \ F , E , d , C
%01111100   %01110111   %01101111   %01111111   \ b , A , 9 , 8
%00000111   %01111101   %01101101   %01100110   \ 7 , 6 , 5 , 4
%01001111   %01011011   %00000110   %00111111   \ 3 , 2 , 1 , 0
%01010010   %10000000   %01000000               \ / , . , -
19 CONSTANTS CIJFERS

\G Bevat de positie van de decimale punt op het display gerekend 
\G vanaf het rechter display. Nul betekent geen decimale punt.
VARIABLE PUNT  ( -- addr )              \ 7segm  Onthoud decimale punt

: ZET-PUNT          ( x1 u -- x2 )      \ 7segm  Neem punt op in code x1 op, geeft x2
    FROM PUNT =  $80 AND  OR            \ maar alleen als u is gelijk aan punt.
    ;                                   \ Als u nul is dan is er geen punt !!

: >DISPLAY          ( addr u -- )       \ 7segm  Stuur u getal digits van RAM-adres
    FOR                                 \ naar het 7-segment display
        @+  &0 -  DUP 9 > IF  7 -  THEN \ Lees en converteer naar index
        3 +  19 UMIN CIJFERS            \ Van index naar 7-segment
        I' ZET-PUNT  >DRIVER
    NEXT  
    DROP  TOON                          \ en laat ze zien !
    ;

\G Stuur het resultaat van een getal conversie 'addr' 'u' met <# # etc. #>
\G naar het zeven segmentdisplay. Als er minder dan #DIGITS cijfers
\G zijn dan wordt het getal links aangevuld met lege digits.
: RTYPE             ( addr u -- )       \ 7segm "ram-type" Type getal op 7-segm. display's
    #DIGITS OVER U> IF                  \ Getal is minder dan #DIGITS lang ?
        #DIGITS OVER - FOR              \ Ja, vul aan met lege digits
          0  OVER I' + ZET-PUNT         \ Neem eventueel de punt op
          >DRIVER                       \ Stuur naar display
        NEXT
        DUP 0= IF  2DROP EXIT  THEN     \ Stop nu als u nul is !!!
    THEN
    #DIGITS UMIN >DISPLAY               \ Type getal van max. #DIGITS
    ;

\ Einde
