\ ==============================================================
\ [TITLE]     AVR ByteForth BAMBOE target library. 
\ [FILE]      BAMBOE.FRT
\ [CATEGORY]  Library 82/188 bytes
\ [VSN]       1.00 
\ [CREATED]   January 16, 1994, Willem Ouwerkerk 
\ [LASTCHNGD] June 30, 2003, Willem Ouwerkerk 
\ [COPYRIGHT] Dutch Forth Users Group (C) 1994-2003
\ [COMPILE]   AVR ByteForth Environments
\ [AUTHOR]    Willem Ouwerkerk 
\ [PURPOSE]   8 bits I/O, Bamboe using 5 io-port bits on port D
\             Example of use of machine code compared to high level
\ ==============================================================

PORTD SFR POORT                         \ PortD of 90S2313

\G Zet de poort-bits voor gebruik met de bamboe.
MACRO: SETUP-BAMBOE  ( -- )             \ Bamboe
    $78 SETDIR POORT
    ;


1 [IF]

$08 CONSTANT #BITS

\ Connections to port P3
\
\ Bit 0 = Free
\ Bit 1 = Free
\ Bit 2 = Free
\ Bit 3 = Load input
\ Bit 4 = Output ready
\ Bit 5 = Clock
\ Bit 6 = Data to Bamboe outputs
\ Bit 7 = Data from Bamboe inputs

\G Stuur de data 'x' naar de bamboe uitgangen.
CODE BAMBOE!  ( x -- )                  \ Bamboe "bamboe-store" At 4 MHz it takes 30 microsec.
    R16 X+ LD,                          \ Pop byte
    PORTD 3 SBI,                        \ Init. (don't sample input bits).
    R17 #BITS LDI,                      \ A Bamboe has #BITS bits
    DO,
        R16 ROL,                        \ Shift to left thru carry
        CS IF,
            PORTD 6 SBI,                \ Set one bit ready
        ELSE,
            PORTD 6 CBI,
        THEN,
        PORTD 5 CBI,                    \ Send data, make clockbit low 
        PORTD 5 SBI,                    \ Set data, make clockbit high
    R17 LOOP,                           \ Do this #BITS times
    PORTD 4 SBI,                        \ Ready, shiftreg. to outputs
    PORTD 4 CBI,                        \ And disconnect them again 
    RET,                                \ End
END-CODE

\G Lees de data 'x' van de bamboe ingangen.
CODE BAMBOE@    ( -- x )                \ Bamboe "bamboe-fetch" At 4 MHz it takes 29 microsec.
    PORTD 3 CBI,                        \ Load (sample) inputs to shiftregister
    PORTD 7 SBI,                        \ Init. (inputbit activated).
    PORTD 3 SBI,                        \ Disconnect inputs.
    R17 #BITS LDI,                      \ Bamboe has #BITS bits
    DO,
        CLC,
        PIND 7 SBIC,                    \ Read one data bit
        SEC,
        PORTD 5 CBI,                    \ Get new databit, make clockbit low
        PORTD 5 SBI,                    \ Make clockbit high
        R16 ROR,                        \ Shift received bit in it to right
    R17 LOOP,                           \ Do this #BITS times
    R16 COM,                            \ Complement all bits
    -X R16 ST,                          \ Push byte
    RET,                                \ End
END-CODE

[ELSE]

 $00 CONSTANT LEES-INGANG              \ Bit patterns for wanted functions
 $18 CONSTANT UITGANG-KLAAR 
 $80 CONSTANT DATA-IN
 $40 CONSTANT DATA-UIT
 $28 CONSTANT KLOK
 $08 CONSTANT NUL

 $08 CONSTANT #BITS                     \ Number of output bits
 
 : SCHRIJF-BIT  ( x -- )                \ Write one bit
    DUP TO POORT  KLOK OR TO POORT 
    ;
 
 : BAMBOE!      ( x -- )                \ Write #BITS bits
    #BITS FOR
        DUP  1 I' 1- LSHIFT  AND 0= 0=
        DATA-UIT AND  NUL OR  SCHRIJF-BIT
    NEXT
    DROP  UITGANG-KLAAR TO POORT 
    ;
 
 : LEES         ( -- x )                \ Read input port
    DATA-IN NUL OR TO POORT  FROM POORT 
    ;

 : LEES-BIT     ( -- x )                \ Leave $80 when input is zero
    LEES DATA-IN AND 0=   $80 AND 
    ;
 
 : BAMBOE@      ( -- x )                \ Read #BITS bamboe input bits
    LEES-INGANG TO POORT  NUL TO POORT
    0  #BITS FOR
        U2/
        LEES-BIT  OR
        NUL TO POORT  KLOK TO POORT
    NEXT 
    ;

[THEN]

\ End
