\ ==============================================================
\ [TITLE]     ByteForth error recovery mechanism
\ [FILE]      CATCH.FRT
\ [CATEGORY]  Library ( 52 bytes )
\ [VSN]       1.00 
\ [CREATED]   October 6, 2000, Willem Ouwerkerk 
\ [LASTCHNGD] November 17, 2001, Willem Ouwerkerk 
\ [COPYRIGHT] De Elektrise Schaar (C) 2000-2001
\ [USES]      AT90Sxxxx
\ [PURPOSE]   Multi level return mechanism for error recovery
\             This file takes one register variable and defines
\             three new macros, one with special-case optimizer.
\ ==============================================================

\G Wijs naar de in gebruik zijnde 'error-handler'.
REGISTER ERROR-HANDLER              \ Catch  Pointer to error frame

\G Zet een 'exception-frame' op de return-stack en voer het token
\G 'xt' uit (net als met EXECUTE). Als er niets fout gaat staat
\G er na uitvoering een 'nul' op de stack, bovenop wat door 'xt'
\G op de stack is aangebracht. Het 'exception-frame' is verwijderd.
\G Zie ook: THROW voor de rest van de beschrijving (20).
$05 MACRO CATCH         ( i*x xt -- j*x 0 | j*x n ) \ Catch
    ZH X+ LD,                       \ Get xt
    ZL X+ LD,
    XL PUSH,                        \ Save datastack
    ADR ERROR-HANDLER PUSH,         \ Push old handler
    ADR ERROR-HANDLER SPL IN,       \ Save new error handler
    ICALL,                          \ Execute xt
    ADR ERROR-HANDLER POP,          \ Restore prev. handler
    R0 POP,                         \ Remove DSP
    R16 0 LDI,                      \ Push zero to stack
    -X R16 ST,
    RET,
END-CODE

SPECIAL EXECUTE CATCH               \ Add existing special case optimizer

\G Haal het 'exception-frame' van de return-stack en ga door met
\G uitvoeren na CATCH. De data- en return-stack zijn teruggebracht in
\G de toestand voor de CATCH met daarbovenop het foutnummer (14).
$10 MACRO THROW          ( k*x n -- k*x || i*x n )  \ Catch
    R16 X+ LD,                      \ Pop error number
    R16 TST,
    NZR IF,
        SPL ADR ERROR-HANDLER OUT,  \ Restore return stack
        ADR ERROR-HANDLER POP,      \ Restore error handler
        XL POP,                     \ Restore datastack
        -X R16 ST,                  \ Push error number on top
    THEN,
    RET,
END-CODE

:SPECIAL THROW                      \ Special case optimizer for THROW
    5 OUTPUT? IF                    \ Literal ?
        REMOVE-LIT  >R  R16 R@ LDI,
        R> 0<> IF
            6  1  EXIT              \ Yes, valid error number !
        ELSE
            -X R16 ST,              \ No, a zero, bull !
        THEN
    THEN
    -1                              \ Standard optimizer may do it
;SPECIAL

\G Maak de data-stack schoon en voer de funktie -1 THROW uit (12).
$05 MACRO ABORT          ( i*x -- || R: j*x -- ) \ Catch
    SPL ADR ERROR-HANDLER OUT,      \ Restore return stack
    ADR ERROR-HANDLER POP,          \ Restore error handler
    XL POP,                         \ Restore datastack
    XL DSP0 DROP LDI,               \ Empty datastack
    R16 255 LDI,                    \ Push  error number on top
    -X R16 ST,
    RET,
END-CODE

\ End
