\ ==============================================================
\ [TITLE]     AVR ByteForth double number routines
\ [FILE]      DOUBLE.FRT
\ [CATEGORY]  Library ( 572 bytes )
\ [VSN]       1.02
\ [CREATED]   November 8, 1996, Willem Ouwerkerk
\ [LASTCHNGD] March 3, 2000, Willem Ouwerkerk
\ [COPYRIGHT] Elektrise Schaar (C) 1996-2000
\ [USES]      AVR ByteForth
\ [PURPOSE]   16 bit arithmetic and comparision routines
\ ==============================================================

ATOM 2SWAP      ATOM 2OVER      ATOM DABS

\G Vermenigvuldig 'd1' en 'd2' met het signed resultaat 'd3'.
: D*                ( d1 d2 -- d3 )     \ Double "d-star"
    2 PICK
    OVER XOR >R
    DABS  2SWAP  DABS  DUM* 2DROP
    R> 0< IF  DNEGATE  THEN
    ;

\G De vlag is true als 'd1' kleiner is dan 'd2'.
: D<                ( d1 d2 -- flag )   \ Double "d-less"
    ROT 2DUP =
    IF    -ROT D- 0<
    ELSE  SWAP < NIP
    THEN  NIP
    ;

\G De vlag is true als 'd1' groter is dan 'd2'.
: D>                ( d1 d2 -- flag )   \ Double "d-greater"
    2SWAP  D<
    ;

\G De vlag is true als 'd' een positief getal groter dan nul is.
: D0>               ( d -- flag )       \ Double "d-zero-greater"
    NIP 0>                              \ Debugged 15-06-99 *WO*
    ;

\G De vlag is true als 'd' kleiner is dan nul.
: D0<               ( d -- flag )       \ Double "d-zero-less"
    NIP 0<
    ;

\G De vlag is true als 'dx' gelijk is aan nul.
: D0=               ( dx -- flag )      \ Double "d-zero-equals"
    OR 0=
    ;

\G De vlag is true als 'dx1' is gelijk is aan 'dx2'.
: D=                ( dx1 dx2 -- flag ) \ Double "d-equals"
    D-  OR 0=
    ;

\G De vlag is true als 'dx1' ongelijk is aan 'dx2'.
: D<>               ( dx1 dx2 -- flag ) \ Double "d-not-equal"
    D-  OR 0= INVERT
    ;

\G High level toegang tot de carry vlag, de 'flag' is true als de
\G carry gezet is.
CODE CARRY          ( -- flag )         \ Double
    R16 R16 SBC,
    -X R16 ST,
END-CODE

\G De vlag is true als 'du1' kleiner is dan 'du2'.
: DU<               ( du1 du2 -- flag ) \ Double "d-u-less"
    D-  OR 0= IF  FALSE EXIT  THEN  CARRY
    ;

\G De vlag is true als 'du1' groter is dan 'du2'.
: DU>               ( du1 du2 -- flag ) \ Double "d-u-greater"
    2SWAP DU<
    ;

\G 'd3' is de grotere van 'd1' en 'd2'.
: DMAX              ( d1 d2 -- d3 )     \ Double "d-max"
    2OVER 2OVER D< IF  2SWAP  THEN  2DROP
    ;

\G 'd3' is de kleinere van 'd1' en 'd2'.
: DMIN              ( d1 d2 -- d3 )     \ Double "d-min"
    2OVER 2OVER D> IF  2SWAP  THEN  2DROP
    ;

\G 'du3' is de grotere van 'du1' en 'du2'.
: DUMAX             ( du1 du2 -- du3 )  \ Double "d-u-max"
    2OVER 2OVER DU< IF  2SWAP  THEN  2DROP
    ;

\G 'du3' is de kleinere van 'du1' en 'du2'.
: DUMIN             ( ud1 ud2 -- ud3 )  \ Double "d-u-max"
    2OVER 2OVER DU> IF  2SWAP  THEN  2DROP
    ;

2VARIABLE ANUMBER  VARIABLE SQRT        \ Work space

\G Bereken de wortel van 'du', met als resultaat 'u'.
: DUSQRT            ( du -- u )         \ Double "d-u-square-root" 
    TO ANUMBER  CLEAR SQRT  $80
    8 FOR
        DUP FROM SQRT +  DUP UM*  
        FROM ANUMBER DU> 0=
        IF  DUP +TO SQRT  THEN  U2/
    NEXT
    DROP  FROM SQRT
    ;

\ Einde
