\ Bestuur grafisch LCD scherm, 122*32 pixels via 6800 interface, 1106 bytes.

PORTA sfr data
PORTD sfr control

PORTD 0 bit-sfr nc1                  \ Ongebruikt
PORTD 1 bit-sfr nc2                  \ Ongebruikt
PORTD 2 bit-sfr l/s                  \ Altijd laag (Ongebruikt)
PORTD 3 bit-sfr actief               \ Laag is start cyclus
PORTD 4 bit-sfr kies2                \ Chip select 2, laag actief
PORTD 5 bit-sfr klok                 \ 2 KHz (via aparte osc.)
PORTD 6 bit-sfr kies1                \ Chip select 1, laag actief
PORTD 7 bit-sfr adres                \ L = instr, H = data

work

Doc umentatie grafische LCD connector

 1 = Massa                          11 = DB1
 2 = Vcc (5 Volt)                   12 = DB2
 3 = LCD contrast (0 tot -5 Volt)   13 = DB3
 4 = Data/Instr.                    14 = DB4
 5 = CS1, laag actief               15 = DB5
 6 = CS2, laag actief               16 = DB6
 7 = Klok (2KHz)                    17 = DB7
 8 = Lees, laag actief              18 = Kies, L = 8051, H = 6800
 9 = Schrijf, laag actief           19 = Led A (+)
10 = DB0                            20 = Led K (-) 

Enddoc

CODE KLOK-AAN   ( -- )                  \ Maak 2 KHz klok op PD.5
    R16 $40 LDI,                        \ Zet Timer 1 uitgang-A in toggle mode
    TCCR1A R16 OUT,
    R16 $09 LDI,                        \ Gebr. 4 MHz klok, en 'auto clear'
    TCCR1B R16 OUT,
    R16 CLR,                            \ Zet timer 1 registers op nul
    TCNT1H R16 OUT,
    TCNT1L R16 OUT,
    R16 1000. NIP LDI,                  \ De cyclus is 250 us, freq = 2KHz
    OCR1AH R16 OUT,
    R16 1000. DROP LDI,
    OCR1AL R16 OUT,
    RET,
END-CODE


\ Basis aansturing van grafisch display

 61 CONSTANT #HALF                      \ Midden van display
122 CONSTANT #RIJ                       \ Aantal kolommen
VARIABLE RIJ                            \ Actieve kolomrij
VARIABLE LIJN                           \ Actieve display regel
VARIABLE ROL                            \ Actieve scroll regel
2VARIABLE LETTERTYPE                    \ Bevat xt van lettertype

\G Zet 'char' op het grafisch LC-display. Het lettertype is
\G afhankelijk van de ingestelde letterset.
: EMIT          ( char -- )             \ Glcd  Emit via een vector
    FROM LETTERTYPE EXECUTE
    ;

\G Stuur de instructie 'byte' naar het hele LC-display.
: INSTRUCTIE    ( byte -- )             \ Glcd  Aan beide chips meegedeeld !!
    CLEAR KIES1    CLEAR KIES2  
    CLEAR ADRES    SET ACTIEF   
    TO DATA  
    CLEAR ACTIEF   SET ADRES  
    SET KIES1      SET KIES2  
    ;

: RESET-DISPL   $E2 INSTRUCTIE ;        \ Init. diplay
: DISPLAY-AAN   $AF INSTRUCTIE ;        \ display aan

: KOLOM         ( +n -- )               \ Naar kolom +n van #RIJ
    DUP #HALF 1- U> IF  #HALF -  THEN  
    $7F AND  INSTRUCTIE
    ;

: REGEL         ( +n -- )               \ Naar regel +n van 4
    3 AND $B8 OR  INSTRUCTIE
    ;

\G Positioneer de cursor op positie 'x' 'y' van het LC-display.
: XY            ( x y -- )              \ Glcd "x-y" Positioneer afbeeldsysteem
    DUP TO LIJN  REGEL
    DUP TO RIJ   KOLOM
    ;

\G Scroll regel '+n' naar de bovenste regel van het LC-display.
: SCROLL        ( +n -- )               \ Glcd  Scroll display (regel +n) bovenaan
    $1F AND  $C0 OR INSTRUCTIE
    ;


\ Nu een karakter aansturing

\G Stuur de bitrij 'x' naar actieve positie op het LC-display.
: >DISPLAY      ( x -- )                \ Glcd "to-display" Zet een bitrij op het display
    FROM RIJ #HALF 1- U> IF             \ Eerste helft display vol ?
        FROM RIJ #RIJ U> IF  DROP EXIT  THEN \ Tweede helft vol, klaar !
        FROM RIJ KOLOM                  \ Hardware op kolom in 2e helft
        CLEAR KIES2                     \ Nee, activeer tweede helft
        SET ACTIEF   TO DATA            \ en stuur bitrij
        CLEAR ACTIEF  INCR RIJ
        SET KIES2    EXIT
    THEN
    CLEAR KIES1                         \ Activeer eerste helft
    SET ACTIEF    TO DATA 
    CLEAR ACTIEF  INCR RIJ
    SET KIES1 
    ;

\G Wis vanaf de cursorpositie tot het eind van de regel.
: WIS           ( -- )                  \ Glcd  Wis (deel) van actieve regel
    #RIJ FROM RIJ -  #RIJ UMIN ?DUP
    IF  FOR  0 >DISPLAY  NEXT  THEN
    ;

\G Zet 'u' karakters vanaf ROM-adres  'addr' op het LC-display.
: TYPE          ( addr u -- )           \ Glcd  Type u karakters vanaf ROM adres a
    FOR  COUNT EMIT  NEXT 2DROP
    ;

\G Zet 'u' karakters vanaf RAM-adres  'addr' op het LC-display.
: RTYPE         ( addr u -- )           \ Glcd "ram-type"  Type u karakters vanaf RAM adres a
    FOR  @+ EMIT  NEXT 2DROP
    ;

\G Zet de cursor linksboven op het LC-display.
: HOME          ( -- )                  \ Glcd  Naar eerste kolom linksboven
    0 0 XY
    ;

\G Wis het LC-display en voer de functie HOME uit.
: PAGE          ( -- )                  \ Glcd  Wis scherm en HOME
    4 FOR
        0 I' 1- XY  WIS
    NEXT
    HOME  0 SCROLL  CLEAR ROL
    ;

\G Ga naar de volgende regel op het LC-display, scroll het display
\G een regel omhoog als er geen lege regels meer beschikbaar zijn.
: CR            ( -- )                  \ Glcd  "c-r" Naar volgende regel of scroll
    FROM LIJN 3 <> IF
        WIS  INCR LIJN  0 FROM LIJN XY
    ELSE
        FROM ROL 1+ 3 AND  
        DUP 8 * SCROLL
        0 FROM ROL XY  WIS  
        0 FROM ROL XY  TO ROL
        3 TO LIJN
    THEN
    ;

\G Zet het bitpatroon 'x' 'u' maal op het LC-display.
: STREEP        ( x u -- )              \ Glcd  Zet u maal bitpatroon x op display
    #RIJ UMIN FOR  DUP >DISPLAY  NEXT  DROP
    ;

\G Maak het LC-display klaar voor gebruik.
: INITIALISEER  ( -- )                  \ Glcd  Maak LCD klaar voor gebruik
    -1 SETDIR CONTROL  -1 SETDIR DATA
    KLOK-AAN SET KIES1  SET KIES2  
    CLEAR L/S  SET ACTIEF  RESET-DISPL
    DISPLAY-AAN PAGE
    ;

\ Einde
