\ ----------------------------------------------------------------------
\ LANGUAGE    : AVR ByteForth vsn 2.00
\ PROJECT     : GP2D02.FRT
\ DESCRIPTION : GP2D02 IR-afstandsensor.
\ CATEGORY    : Applicatie, afmeting: 108 bytes.
\ AUTHOR      : Willem Ouwerkerk, Mei 29, 2003
\ LAST CHANGE : Willem Ouwerkerk, Mei 29, 2003
\ ----------------------------------------------------------------------

\ Uitlees en test routine voor Sharp GP2D02 sensor, W.O. 3 dec. 1999
\ Aangepast aan ByteForth 28-11-00, lengte 204/162 bytes.
\ Ook gebruikt om Mini US afstandmeter te testen GP2D02 interface.
\
\ De GP2D02 sensor geeft slechts een relatieve maat voor de afstand.
\ Enkele voorbeelden gemaakt met behulp van de routine TEST vertellen
\ meer daarover, links afstand en rechts de uitkomsten:
\
\ Afstand       Wit papier      Mijn hand       Donker grijs
\ -----------------------------------------------------------
\  8 cm              11              24              10
\ 10 cm              17              35              25
\ 20 cm             110             115             112
\ 30 cm             142             140             140
\ 40 cm             156             159             154
\ 50 cm             162             160             168
\ 60 cm             167             166             185
\ 70 cm             171             170             190
\ 80 cm             174             176             200
\ meer dan ..       200             200             200
\
\
\ De Mini US afstandmeter geeft bij benadering de afstand in CM terug.

PORTB 4 BIT-SFR DATA                \ PB.4 is Vout
PORTB 5 BIT-SFR CONTROL             \ PB.5 is Vin

\G Initialiseer I/O-pennen voor de GP2D02.
: SETUP-GP2D02  ( -- )              \ GP2D02
    FALSE SETDIR DATA
    TRUE SETDIR CONTROL
    ;

MACRO: VOLGEND-BIT  ( x1 -- x2 )    \ Lees een databit
    SET CONTROL  1 /MS  CLEAR CONTROL
    1 /MS  2*  FROM DATA 0= 1 AND  OR
    ;

\G Geef 'afstand' tot een object aan (lager is dichterbij).
\G Een meting duurt ongeveer 75 milliseconden.
: AFSTAND           ( -- afstand )  \ GP2D02 Lees afstand uit GP2D02 sensor
    SET DATA  CLEAR CONTROL         \ activeer sensor en
    70 MS                           \ wacht even
    0  8 FOR                        \ begingetallen op de stack, neem
        VOLGEND-BIT                 \ acht maal een bit op
    NEXT
    SET CONTROL  2 MS               \ deactiveer sensor
    ;

