\ ----------------------------------------------------------------------
\ LANGUAGE    : AVR ByteForth
\ PROJECT     : I2C temperatuur driver network
\ DESCRIPTION : Measuring temperature with LM75CIM
\ CATEGORY    : Application, size: 516 bytes.
\ AUTHOR      : Willem Ouwerkerk, November 25, 1999-2000
\ LAST CHANGE : Willem Ouwerkerk, March 4, 2000
\ ----------------------------------------------------------------------

NEEDS I2C-PRIM.FRT

$90 CONSTANT LM75-ID                \ LM75 basis adres

: SELECT-CHIP   ( chip -- dev-id )  \ Max. 8 chips
    7 AND 2*  LM75-ID  +
    ;

: REGISTER      ( reg chip -- )     \ Selecteer LM75 register
    START-BIT
    SELECT-CHIP  BYTE-UIT
    3 AND BYTE-UIT
    STOP-BIT
    ;

\G Vraag de tenperatuur 'd' van de chip 'chip'. De temperatuur 
\G wordt afgegeven in halve graden per bit, waar 0 nul graden 
\G is, en het bereik loopt van -55 tot +125 graden.
: TEMPERATUUR   ( chip -- d )       \ I2C-LM75  Lees temperatuur als
    START-BIT                       \ 9 bits getal
    SELECT-CHIP  1 OR  BYTE-UIT
    BYTE-IN  ACK-BIT  >R            \ msb
    BYTE-IN  NACK-BIT  STOP-BIT     \ lsb
    7 RSHIFT  R@ 2*  OR             \ Zet temperatuur om naar
    R> 7 LSHIFT                     \ een Forth dubbel getal
    ;

\G Zet 'x' in het configuratie register van 'chip'.
: CONFIGURATIE  ( x chip -- )       \ I2C-LM75  Zet LM75 configuratie
    >R  1 R@ REGISTER               \ Selecteer config. reg.
    START-BIT
    R@ SELECT-CHIP  BYTE-UIT
    BYTE-UIT  STOP-BIT              \ Zet config. register
    0 R> REGISTER                   \ Selecteer temp. reg.
    ;

: >TEMPERATUUR  ( +d -- d )         \ I2C-LM75  Converteer Forth dubbel
    7 LSHIFT  OVER 1 RSHIFT  OR     \ naar LM75 temperatuur
    SWAP  7 LSHIFT  SWAP
    ;

\G Zet de ondergrens 'd' van de thermostaatfunctie van 'chip'.
: ONDERGRENS    ( d chip -- )       \ I2C-LM75  
    >R  2 R@ REGISTER               \ Selecteer THYST reg.
    START-BIT
    R@ SELECT-CHIP  BYTE-UIT
    >TEMPERATUUR  BYTE-UIT          \ Zet thermostaat
    BYTE-UIT  STOP-BIT              \ ondergrens
    0 R> REGISTER                   \ Selecteer temp. reg.
    ;

\G Zet de bovengrens 'd' van de thermostaatfunctie van 'chip'.
: BOVENGRENS    ( dx chip -- )      \ I2C-LM75  
    >R  3 R@ REGISTER               \ Selecteer TOS reg.
    START-BIT
    R@ SELECT-CHIP  BYTE-UIT
    >TEMPERATUUR  BYTE-UIT          \ Zet thermostaat
    BYTE-UIT  STOP-BIT              \ bovengrens
    0 R> REGISTER                   \ Selecteer temp. reg.
    ;

                               ( End )
