\ I2C primitieven voor een 4 MHz kristal (218 bytes)

PORTD 4  BIT-SFR SDA                   \ Serieele data lijn
PORTD 5  BIT-SFR SCL                   \ Serieele klok lijn

\ Algemene I2C primitieve routines

4 CRYSTAL? [IF]

\G Wacht ongeveer 4.7 microsec. bij een 4 MHz kristal.
\G Er zijn versies voor 4, 8, 10 en 11.059 MHz.
: WACHT          ( -- )                 \ I2C-Prim
    2 FOR  NEXT
    ;

[THEN]
8 CRYSTAL? [IF]                         \ 8 MHz versie
: WACHT          ( -- ) 8 FOR  NEXT  ;
[THEN]
10 CRYSTAL? [IF]                        \ 10 Mhz versie
: WACHT          ( -- ) 10 FOR  NEXT ;
[THEN]
11 CRYSTAL? [IF]                        \ 11.096 Mhz versie
: WACHT          ( -- ) 10 FOR  NEXT ;
[THEN]

\G Initialiseer de I2C bus.
: SETUP-I2C      ( -- )                 \ I2C-Prim  Setup I2C I/O bits
    SET SDA  SET SCL
    1 SETDIR SDA  1 SETDIR SCL
    ;

\G Genereer een I2C startconditie.
: START-BIT     ( -- )                  \ I2C-Prim  Genereer startbit
    SET SDA  SET SCL  
    CLEAR SDA  WACHT
    CLEAR SCL  WACHT
    ;

\G Genereer een I2C stopconditie.
: STOP-BIT      ( -- )                  \ I2C-Prim  Genereer stopbit
    CLEAR SDA  SET SCL  WACHT  SET SDA
    ;

\G Genereer een I2C negatieve acknowledge.
: NACK-BIT      ( -- )                  \ I2C-Prim  Genereer no-ackbit
    SET SDA  SET SCL  WACHT  CLEAR SCL  WACHT
    ;

\G Genereer een I2C acknowledge.
: ACK-BIT       ( -- )                  \ I2C-Prim  Genereer ackbit
    CLEAR SDA  SET SCL  WACHT
    CLEAR SCL  WACHT  SET SDA
    ;

\G Test het inkomende bit. Geef true als het een acknowledge is
\G en anders false.
: ACK?          ( -- vlag )             \ I2C-Prim  Test binnenkomend ackbit, vlag is
    SET SDA  SET SCL  0 SETDIR SDA
    WACHT  FROM SDA 0=  1 SETDIR SDA    \ true als er een ack ontvangen is
    CLEAR SCL  WACHT
    ;

\G Verzend byte 'x' via de I2C bus.
: (BYTE-UIT)    ( x -- )                \ I2C-Prim  Stuur byte
    8 FOR
        DUP 128 AND  TO SDA  2*
        SET SCL  WACHT  CLEAR SCL
    NEXT
    DROP
    ;

\G Verzend byte 'x' via de I2C bus en genereer niets als de 
\G geadresseerde device niet reageert met een acknowledge.
: BYTE-UIT      ( x -- )                \ I2C-Prim  Stuur byte en test ack
    (BYTE-UIT)  ACK? DROP
    ;

\G Ontvang byte 'x' via de I2C bus.
: BYTE-IN       ( -- x )                \ I2C-Prim  Ontvang byte
    0 SETDIR SDA
    0  8 FOR
        SET SCL  WACHT 
        2*  FROM SDA 1 AND  OR  
        CLEAR SCL  WACHT
    NEXT
    1 SETDIR SDA
    ;

\G De vlag is true als het device geadresseerd met 'dev-id'
\G aanwezig is op de I2C bus en anders is hij false.
: AANWEZIG?     ( dev-id -- vlag )      \ I2C-Prim  De vlag is true als dev-id aanwezig is
    START-BIT  (BYTE-UIT)  ACK?  STOP-BIT
    ;

\ Einde