\ ==============================================================
\ [TITLE]     AVR ByteForth keyboard library. 
\ [FILE]      KEYB1.FRT
\ [CATEGORY]  Library ( 364 bytes )
\ [VSN]       1.00 
\ [CREATED]   April 10, 1995, Willem Ouwerkerk 
\ [LASTCHNGD] May 30, 2003, Willem Ouwerkerk 
\ [COPYRIGHT] De Elektrise Schaar (C) 1995-2003
\ [USES]      AVR ByteForth cross compiler
\ [PURPOSE]   Reading a key from a 13-keys dump telephone keyboard
\             Connected in a row column matrix. A keyboard like
\             this can be bought for example for 2,- at any dump
\ ==============================================================

\            Connected in a row column matrix, like this:
\
\            Keyboard at port-1
\
\            bit of port-1 0 - 1 - 2 - 3
\                          |   |   |   |
\                     4----1---4---7---* 
\                          |   |   |   |
\                     5----2---5---8---0
\                          |   |   |   |
\                     6----3---6---9---#
\
\            How are the bits used:
\
\            Bit 0 = Row with 1 2 3 keys
\            Bit 1 = Row with 4 5 6 keys
\            Bit 2 = Row with 7 8 9 keys
\            Bit 3 = Row with * 0 # keys
\            Bit 4 = Column with 1 4 7 * keys
\            Bit 5 = Column with 2 5 8 0 keys
\            Bit 6 = Column with 3 6 9 # keys
\            Bit 7 = R key

PORTB SFR PORT                          \ Access port-B

\ Leave true if the R key is pressed on the keyboard
: RKEY          ( -- Flag )             \ Keyb1 'R' key pushed ?
    FROM PORT $80 AND 0=
    ;

\ Get column number of the pressed key, leave -1 if no key
\ is pressed.
: COLUMN        ( -- +n|-1 )            \ Keyb1 Get column of pushed key
    FROM PORT INVERT $F AND
    4  0
    DO      I  OVER 1 I LSHIFT  =
            IF 
                NIP  UNLOOP EXIT
            ELSE
                DROP
            THEN
    LOOP    DROP  -1
    ;

\ Get row and column of the pressed key.
: SCAN-KEYB     ( -- Row Column )       \ keyb1  Get row column of pushed key
    $70 SETDIR PORT                     \ Bit 4, 5 and 6 are output,
    3  0
    DO      $10 I LSHIFT  INVERT TO PORT
            I 1+  FROM PORT $F AND $F <
            IF      
                COLUMN  UNLOOP EXIT
            ELSE
                DROP
            THEN
    LOOP    -1  -1
    ;

\ Get the key's scan code on the stack, leave -1 if no key is
\ pressed include the R-key.
: SCAN-KEY      ( -- 0to12|-1)          \ keyb1  Get scan code of a key
    SCAN-KEYB S>D
    IF      2DROP  RKEY
            IF      0   EXIT
            THEN    -1  EXIT
    THEN    3 *  +
    ;

\ Hold scan code of latest pressed key.
VARIABLE KEY-NR                         \ Keyb1  Hold scan code

\G Geef een true vlag als een toets ingedrukt is, anders false.
: AKEY?         ( -- Flag )             \ Keyb1 "a-key-question"
    SCAN-KEY  DUP TO KEY-NR  0< 0=      \ Any key pushed ?
    ;

\ Define a translation table, so that the ASCII values printed
\ on the keys correspond to the value returned by AKEY .
&# &0 &* &9 &8 &7 &6 &5 &4 &3 &2 &1 &R
13 CONSTANTS KEY-TABLE                  \ Keyb1

\G Wacht tot er een toets ingedrukt is, char is de ASCII waarde 
\G van die toets.
: AKEY          ( -- char )             \ Keyb1 "a-key"
    BEGIN   AKEY?  50 MS                \ Key pushed down ?
            AKEY?  AND                  \ Are we shure ?
    UNTIL
    FROM KEY-NR  KEY-TABLE              \ Get ASCII value of key
\   BEGIN  50 MS  AKEY? 0= UNTIL        \ Accept single keystrokes only
    ;

\ End
