\ ==============================================================
\ [TITLE]     AVR ByteForth keyboard library. 
\ [FILE]      KEYB2.FRT
\ [CATEGORY]  Pincode lock, 290 bytes
\ [VSN]       2.00
\ [CREATED]   October 5, 1996, Willem Ouwerkerk 
\ [LASTCHNGD] May 30, 2003, Willem Ouwerkerk 
\ [COPYRIGHT] De Elektrise Schaar (C) 1996-2003
\ [USES]      AVR ByteForth cross compiler
\ [PURPOSE]   Reading a key from a 12-keys dump telephone keyboard
\             Connected in a row column matrix. A keyboard like
\             this can be bought for 5,- at CONRAD electronics.
\ ==============================================================

\            Keys are connected in a row column matrix, like this:
\
\            Keyboard connected to port-1
\
\                          1   2   3   4  column number
\               row mask   1   2   4   8  column mask
\         row number       |   |   |   |
\                  0   16--1---4---7---* 
\                          |   |   |   |
\                  1   32--2---5---8---0
\                          |   |   |   |
\                  2   64--3---6---9---#
\
\            How are the bits used:
\
\            Bit 0 = Row with 1 2 3 keys
\            Bit 1 = Row with 4 5 6 keys
\            Bit 2 = Row with 7 8 9 keys
\            Bit 3 = Row with * 0 # keys
\            Bit 4 = Column with 1 4 7 * keys
\            Bit 5 = Column with 2 5 8 0 keys
\            Bit 6 = Column with 3 6 9 # keys
\
\ Principle version of keyboard routine
\ : KEY   ( -- x )        ROW# 4 *  COLUMN# + ;

ATOM INLINE$                            \ Import string primitive

PORTB SFR PORT                          \ Access port-B

: COLUMN    ( rowmask -- column# )          \ Test keyboard row
    $F0 SETDIR PORT                         \ High four bits are output
    INVERT TO PORT  FROM PORT INVERT 15 AND \ Select row
    4 0 DO  DUP 1 I LSHIFT AND              \ Test a column, key active ?
            IF  DROP I 1+ UNLOOP EXIT  THEN \ Yes, leave column number
    LOOP    DROP FALSE ;                    \ No key in this row pressed

: KEYBOARD  ( -- key|quiet )                \ Read key from keyboard
    64 COLUMN ?DUP IF  8 +  EXIT  THEN      \ Row 3
    32 COLUMN ?DUP IF  4 +  EXIT  THEN      \ Row 2
    16 COLUMN ;                             \ Row 1

: TRANSLATE ( +n -- char )                  \ Convert keycode to char
    1- S>D  S" 147*2580369#" DROP D+ ROM@ ; \ for comfort

\G Geef een true vlag als een toets ingedrukt is, anders false.
: KEY?      ( -- flag )                     \ Keyb2 "key-question" True if a key is pressed
    KEYBOARD 0> ;

\G Wacht tot er een toets ingedrukt is, 'char' is de ASCII waarde 
\G van die toets.
: KEY       ( -- char )                     \ Keyb2 Wait for a key press
    BEGIN  
        KEY?  30 MS  KEY?  AND              \ Any key pressed
    UNTIL
    KEYBOARD  TRANSLATE                     \ Get key and convert to char
    BEGIN  KEY? 0= UNTIL ;                  \ Wait for key release

\ End

