\ ==============================================================
\ [TITLE]     AVR ByteForth MIDI target library. 
\ [FILE]      MIDI.FRT
\ [CATEGORY]  Library, 164 bytes
\ [VSN]       1.00 
\ [CREATED]   Februari 24, 2000, Willem Ouwerkerk 
\ [LASTCHNGD] May 30, 2003, Willem Ouwerkerk 
\ [COPYRIGHT] De Elektrise Schaar (C) 2000-2003
\ [USES]      AVR ByteForth cross compiler
\ [PURPOSE]   MIDI I/O on target AVR with use of baudrate gen.
\ ==============================================================

4 CRYSTAL?  [IF]

\ All baudrates mentioned are for a 4 MHz crystal

\G Initialiseer de UART op een MIDI baudrate van 31K25 baud.
CODE SETUP-MIDI                         \ MIDI
    R16 7 LDI,                          \ Select 31K25 baud
    UBRR R16 OUT,
    R16 $18 LDI,
    UCR R16 OUT,                        \ Enable receiver & transmitter
    RET,                                \ Reset interrupt, set 8 bit data.
END-CODE

[THEN]

8 CRYSTAL?  [IF]

\ All baudrates mentioned are for a 8 MHz crystal,
CODE SETUP-MIDI
    R16 15 LDI,                         \ Select 31K25 baud
    UBRR R16 OUT,
    R16 $18 LDI,
    UCR R16 OUT,                        \ Enable receiver & transmitter
    RET,                                \ Reset interrupt, set 8 bit data.
END-CODE

[THEN]

11 CRYSTAL? [IF]

\ All baudrates mentioned are for a 11.059 MHz crystal.
CODE SETUP-MIDI
    R16 21 LDI,                         \ Select 31K25 baud
    UBRR R16 OUT,
    R16 $18 LDI,
    UCR R16 OUT,                        \ Enable receiver & transmitter
    RET,                                \ Reset interrupt, set 8 bit data.
END-CODE

[THEN]

\G Verzend het karakter 'char' via MIDI.
CODE MIDI-EMIT     ( char -- )         \ MIDI Char to MIDI
    R16 X+ LD,
    BEGIN,
        USR 5 SBIS,
    AGAIN,
    UDR R16 OUT,
    RET,
END-CODE

\G Verzend 'u' karakters vanaf ROM adres 'addr' via MIDI.
CODE MIDI-TYPE     ( addr u -- )       \ MIDI Rom string to MIDI
    R16 X+ LD,
    ZH X+ LD,
    ZL X+ LD,
    R16 TST,
    NZR IF,
        DO,     BEGIN,
                    USR 5 SBIS,
                AGAIN,
                LPM,
                UDR R0 OUT,
                ZL 1 ADIW,
        R16 LOOP,
    THEN,
    RET,
END-CODE

\G Verstuur 'u' karakters vanaf RAM adres 'addr' via MIDI.
CODE MIDI-RTYPE    ( addr u -- )       \ MIDI "midi-ram-type" Ram string to MIDI
    R16 X+ LD,
    ZH X+ LD,
    ZL X+ LD,
    R16 TST,
    NZR IF,
        DO,     BEGIN,
                    USR 5 SBIS,
                AGAIN,
                R17 Z+ LD,
                UDR R17 OUT,
        R16 LOOP,
    THEN,
    RET,
END-CODE

\G Geef true als een karakter via MIDI ontvangen is, anders false.
USR 7 BIT-SFR MIDI-KEY?  ( -- flag )   \ MIDI "midi-key-question" Char present from MIDI

\G Wacht tot een karakter 'char' via MIDI ontvangen is.
CODE MIDI-KEY      ( -- char )         \ MIDI Read char from MIDI
    BEGIN,
        USR 7 SBIS,
    AGAIN,
    R16 UDR IN,
    -X R16 ST,
    RET,
END-CODE

\ End
