\ ==============================================================
\ [TITLE]     AVR ByteForth music control
\ [FILE]      MUSIC.FRT
\ [CATEGORY]  Application ( 232 bytes )
\ [VSN]       1.10 
\ [CREATED]   June 19, 1995, Willem Ouwerkerk
\ [LASTCHNGD] March 3, 2000, Willem Ouwerkerk
\ [COPYRIGHT] Elektrise Schaar (C) 1995-2000
\ [USES]      AVR ByteForth cross compiler
\ [PURPOSE]   Making music with a 4 MHz xtal !!!!!!
\ ==============================================================

REGISTER NOTE-TIME                      \ Length of note timer
REGISTER TEMPO                          \ Length of a note
PORTB 0 BIT-SFR OUT                     \ Music output bit

CODE PERIOD         ( -- )              \ Delay 10.25 us to 2600 us (1-255)
    R17 R16 MOV,
    DO, R18 12 LDI,
        DO, R18 LOOP,
    R17 LOOP,
    RET,
END-CODE

CODE SQUARE         ( period -- )       \ Squarewave with a period time of
    R16 X+ LD,                          \ period * 10 us
    ADR NOTE-TIME CLR,
    BEGIN,  ADR OUT SBI,
            ' PERIOD RCALL,
            ADR OUT CBI,
            ' PERIOD RCALL,
            ADR NOTE-TIME ADR TEMPO CP,
    UGE UNTIL,
    RET,
END-CODE

\G Wacht TEMPO milliseconden.
: REST              ( -- )              \ Music Silence for TEMPO millisec.
    TEMPO MS ;

\G Zet TEMPO zo dat hele noten gespeeld worden.
\G Er zijn ook: 1/2, 1/4 en 1/8 tempo's.
: 1/1       250 TO TEMPO ;   ( -- )     \ Music "one-slash-one"

: 1/2       125 TO TEMPO ;
: 1/4        63 TO TEMPO ;
: 1/8        31 TO TEMPO ;

\G Speel de noot A1 met de lengte van TEMPO millisec. Er zijn nog 22
\G andere noten, die samen ongeveer 2.3 octaaf vormen van A1 tot C3.
: A1        227 SQUARE ;     ( -- )     \ Music "a-one" 2.3 octave

: B1        200 SQUARE ;
: C1        192 SQUARE ;
: D1        172 SQUARE ;
: E1        152 SQUARE ;
: F2        143 SQUARE ;
: FIS2      135 SQUARE ;
: G2        128 SQUARE ;
: GIS2      119 SQUARE ;
: A2        114 SQUARE ;
: AIS2      106 SQUARE ;
: B2        102 SQUARE ;
: C2         96 SQUARE ;
: CIS2       91 SQUARE ;
: D2         85 SQUARE ;
: DIS2       81 SQUARE ;
: E2         76 SQUARE ;
: F3         71 SQUARE ;
: G3         64 SQUARE ;
: A3         57 SQUARE ;
: B3         51 SQUARE ;
: C3         48 SQUARE ;

\G Initialiseer de muziek hardware.
CODE SETUP-MUSIC  ( -- )                \ Music Install timer 0
    R16 2 LDI,
    TCCR0 R16 OUT,                      \ Clock/8 = 2us
    TIMSK R16 OUT,                      \ Overflow 0 int. active
    R16 55 LDI,                         \ Count 200 ticks
    TCNT0 R16 OUT,
    SEI,                                \ Interrupts on
    RET,
END-CODE

CODE NOTETIME                           \ Interrupt code (3.75 us)
    R16 PUSH,
    R17 PUSH,
    R17 SREG IN,
    R16 55 LDI,                         \ Reload count 200 ticks
    TCNT0 R16 OUT,
    ADR NOTE-TIME INC,                  \ Increased every millisec.
    SREG R17 OUT,
    R17 POP,
    R16 POP,
    RETI,
END-CODE  T0-OVERFLOW

DOC Demo program

: OCTAAF                                \ Demo word
    REST F2 FIS2 G2 GIS2 A2 AIS2 B2 C2 CIS2 D2 DIS2 E2 REST
    ;

: MUSIC-DEMO    ( -- )
    SETUP  SETUP-MUSIC  -1 SETDIR OUT
    BEGIN 
        1/8 OCTAAF
        1/4 OCTAAF
        1/2 OCTAAF
        1/1 OCTAAF
        1/2 OCTAAF
        REST A1  REST A2  REST A3
        REST C1  REST C2  REST C3
    AGAIN
    ;  MAIN

ENDDOC

\ End
