\ ==============================================================
\ [TITLE]     AVR ByteForth number output library. 
\ [FILE]      NUMBERS.FRT
\ [CATEGORY]  Library ( ... bytes )
\ [VSN]       1.10 
\ [CREATED]   April 10, 1995, Willem Ouwerkerk 
\ [LASTCHNGD] Februari 22, 2002, Willem Ouwerkerk 
\ [COPYRIGHT] De Elektrise Schaar (C) 1995-2002
\ [USES]      AVR ByteForth cross compiler
\ [PURPOSE]   Number output to current output device
\             Needs: LCD.FRT or RS232.FRT
\ ==============================================================

\G addr is het adres van de cel die het grondtal voor de getal
\G conversie bevat, geldige waarden zijn 2 t/m 36.
VARIABLE BASE      ( -- addr )          \ Numbers Hold number conversion base

VARIABLE HLD                            \ Work addr. in number conv. buffer
#10 CONSTANT NUM                        \ Length of number conversion buffer
NUM VARIABLES CONV                      \ Number string buffer
ADR CONV 2CONSTANT NUMSTART             \ Start of string buffer

\ Unsigned devide routine

\G Deel 'du1' door het getal in 'BASE' tot 'du2' en 'char'. Wordt gebruikt
\G om een digit te reduceren uit een unsigned dubbel getal, tot zijn
\G equivalente ASCII karakter.
CODE DIG-OUT ( du1 -- du2 char )        \ Numbers "dig-out" Divide ud1 by [BASE] to ud2 with char
    R16 X+ LD,
    R17 X+ LD,
    R18 CLR,
    R19 ADR BASE [Y] LD,                \ Base is divisor
    R20 16 LDI,                         \ 16 bits shift
    DO,
        SEC,                            \ Shift in a one
        R17 ROL,                        \ Shift lowbyte
        R16 ROL,                        \ Shift highbyte
        R18 ROL,                        \ Save highbit in workregister
        R18 R19 SBC,                    \ Subtract base
        CS IF,                          \ Unsuccessfull ?
            R17 DEC,                    \ Yes, correct one
            R18 R19 ADD,                \ Restore work register
        THEN,
    R20 LOOP,
    R18 11 CPI,                         \ Digit 10 U> ?
    UGE IF,
        R18 7 ADDI,                     \ Yes, correct 10 to "A"
    THEN,
    R18 &0 ADDI,                        \ Convert to ASCII
    -X R17 ST,
    -X R16 ST,
    -X R18 ST,
    RET,
END-CODE


\ Number conversion primitives

\G Voeg het karakter 'char' toe aan het begin van de uitvoerstring
\G in wording. Te gebruiken tussen <# en #>.
: HOLD      DECR HLD  FROM HLD TO CONV ;                ( char -- )     \ Numbers

\G Als 'n' negatief is, voeg dan een minteken toe aan de uitvoerstring
\G in wording. Te gebruiken tussen <# en #>.
: SIGN      0< IF  [CHAR] -  HOLD  THEN ;               ( n -- )        \ Numbers

\G Begin een nieuwe getal uitvoerstring.
: <#        NUM TO HLD ;                                ( -- )          \ Numbers "less-number-sign"

\G Gooi 'd' weg en maak de uitvoerstring beschikbaar als 'addr' 'u'.
: #>        2DROP  NUMSTART FROM HLD M+  NUM FROM HLD - ; ( d -- addr u ) \ Numbers "number-sign-greater"

\G Deel 'd1' door het getal in 'BASE' met als resultaat 'd2'. De rest 'n'
\G wordt omgezet tot een karakter welke toegevoegd wordt aan het 
\G begin van de uitvoerstring. Te gebruiken tussen <# en #>.
: #         DIG-OUT HOLD ;                              ( d1 -- d2 )    \ Numbers "number-sign"

\G Converteer alle digits in 'd1' als beschreven bij # todat het
\G resulterende quotient 'd2' nul is. Te gebruiken tussen <# en #>.
: #S        BEGIN  # 2DUP OR 0= UNTIL ;                 ( d1 -- d2 )    \ Numbers "number-sign-s"

\ Number output words

\G Druk 'd' af in een vrij formaat.
: D.        TUCK DABS <# #S  ROT SIGN #>  RTYPE SPACE ; ( d -- )        \ Numbers "d-dot"

\G Druk 'n' af in een vrij formaat.
: .         S>D  D. ;                                   ( n -- )        \ Numbers "dot"

\G Druk 'u' af in een vrij formaat.
: U.        0  D. ;                                     ( u -- )        \ Numbers "u-dot"

\G Zet het grondtal op 10.
: DECIMAL   #10 TO BASE ;                               ( -- )          \ Numbers

\G Zet het grondtal op 16.
: HEX       $10 TO BASE ;                               ( -- )          \ Numbers

\ End
