\ ==============================================================
\ [TITLE]     AVR ByteForth two pseudo random generators
\ [FILE]      RANDOM.FRT
\ [CATEGORY]  Library ( 238 or 64 bytes )
\ [VSN]       1.02
\ [CREATED]   November 12, 1996, Willem Ouwerkerk 
\ [LASTCHNGD] March 3, 2000, Willem Ouwerkerk 
\ [COPYRIGHT] De Elektrise Schaar (C) 1996-2000
\ [USES]      AVR ByteForth
\ [PURPOSE]   Random number generators
\ ==============================================================

0 [IF]                                  \ Random according Leo Brodie

2VARIABLE RND                           \ Work location for random numbers

ATOM DABS                               \ Import a macro

: D*                ( d1 d2 -- d3 )     \ Rnd "Double-star"
    2 PICK
    OVER XOR >R
    DABS  2SWAP  DABS  DUM* 2DROP
    R> 0< IF  DNEGATE  THEN
    ;

: RANDOM            ( -- ud )           \ Rnd Make random number ud
    FROM RND 31421. D*  6927. D+  2DUP TO RND
    ;

: CHOOSE            ( u1 -- u2 )        \ Rnd Make random number u2 the
    RANDOM DROP  UM* NIP                \ range of u2 is 0 to u1-1
    ;

: SETUP-RANDOM      ( -- )              \ Rnd Initialise random work location
    31414. TO RND
    ;

[ELSE]                                  \ Shiftregister with XOR

REGISTER RNDL                           \ Random work area
REGISTER RNDH

CODE RANDOM         ( -- u )            \ Rnd Make random number u
    R16 ADR RNDH MOV,
    R16 ROL,                            \ Move bit 14 to bit 15 location
    R16 ADR RNDL EOR,                   \ Xor it with bit 7 of word
    R16 ROL,                            \ Save result in carry
    ADR RNDL ROL,                       \ Shift it in at bit 0 position
    ADR RNDH ROL,                       \ Shift bit 7 to bit 8 position
    -X ADR RNDL ST,                     \ We use low byte as result
    RET,
END-CODE

\G Maak het random getal 'u2', 'u2' ligt in het bereik van 0 tot 'u1-1'.
: CHOOSE            ( u1 -- u2 )        \ Rnd Make random number u2 the
    RANDOM  UM* NIP                     \ range of u2 is 0 to u1-1
    ;

\G Initialiseer het random getal zaadje.
MACRO: SETUP-RANDOM ( -- )              \ Rnd Initialise random work location
    31414. NIP  TO RNDH  
    31414. DROP TO RNDL
    ;

[THEN]

\ End
