\ ==============================================================
\ [TITLE]     AVR ByteForth RS232 target library. 
\ [FILE]      RS232m.FRT
\ [CATEGORY]  Library 164 bytes
\ [VSN]       1.00 
\ [CREATED]   Februari 24, 2000, Willem Ouwerkerk 
\ [LASTCHNGD] Augustus 2, 2002, Willem Ouwerkerk 
\ [COPYRIGHT] De Elektrise Schaar (C) 2000-2002
\ [USES]      AVR ByteForth cross compiler
\ [PURPOSE]   RS232 I/O on ATmega target with use of timer 1.
\ ==============================================================

UCSRA 5 BIT-SFR UDRE

1 CRYSTAL?  [IF]

\ All baudrates mentioned are for a 1 MHz crystal or internal RC-osc.

\G Initialiseer de UART op een RS232 baudrate van 9600 baud.
\G Er zijn versies voor 1, 4, 8 en 11.059 HHz.
CODE SETUP-RS232  ( -- )                \ RS232
\   R16 51 LDI,                         \ Select 2400 baud
\   R16 25 LDI,                         \ Select 4800 baud
    R16 12 LDI,                         \ Select 9600 baud
    R17 00 LDI,
    UBRRH R17 OUT,
    UBRRL R16 OUT,
    R16 2 LDI,                          \ Set U2X
    UCSRA R16 OUT,
    R16 $18 LDI,
    UCSRB R16 OUT,                      \ Enable receiver & transmitter
    RET,                                \ Reset interrupt, set 8 bit data.
END-CODE

[THEN]


4 CRYSTAL?  [IF]

\ All baudrates mentioned are for a 4 MHz crystal

\ Initialiseer de UART op een RS232 baudrate van 9600 baud.
CODE SETUP-RS232                        \ RS232
\   R16 103 LDI,                        \ Select 2400 baud
\   R16 51 LDI,                         \ Select 4800 baud
    R16 25 LDI,                         \ Select 9600 baud
    R17 00 LDI,
    UBRRH R17 OUT,
    UBRRL R16 OUT,
    R16 $18 LDI,
    UCSRB R16 OUT,                      \ Enable receiver & transmitter
    RET,                                \ Reset interrupt, set 8 bit data.
END-CODE

[THEN]

8 CRYSTAL?  [IF]

\ All baudrates mentioned are for a 8 MHz crystal, for
\ a 4 MHz crystal the baudrates are 4800 or 9600 baud.

CODE SETUP-RS232
\   R16 207 LDI,                        \ Select 2400 baud
\   R16 103 LDI,                        \ Select 4800 baud
    R16 51 LDI,                         \ Select 9600 baud
    R17 00 LDI,
    UBRRH R17 OUT,
    UBRRL R16 OUT,
    R16 $18 LDI,
    UCSRB R16 OUT,                      \ Enable receiver & transmitter
    RET,                                \ Reset interrupt, set 8 bit data.
END-CODE

[THEN]

11 CRYSTAL? [IF]

\ All baudrates mentioned are for a 11.059 MHz crystal.
\ The max. baudrate is 115200 with this crystal.

CODE SETUP-RS232
\   R16 287. DROP LDI,                  \ Select 2400 baud
\   R17 287. NIP LDI,

\   R16 143 LDI,                        \ Select 4800 baud
\   R17 00 LDI,

    R16 71 LDI,                         \ Select 9600 baud
    R17 00 LDI,
    UBRRH R17 OUT,
    UBRRL R16 OUT,
    R16 $18 LDI,
    UCSRB R16 OUT,                      \ Enable receiver & transmitter
    RET,                                \ Reset interrupt, set 8 bit data.
END-CODE

[THEN]

\G Verzend het karakter 'char' via de RS232.
CODE RS232-EMIT     ( char -- )         \ RS232 Char to RS232
    R16 X+ LD,
    BEGIN,
        ADR UDRE SBIS,
    AGAIN,
    UDR R16 OUT,
    RET,
END-CODE

\G Verzend 'u' karakters vanaf ROM adres 'addr' via de RS232.
CODE RS232-TYPE     ( addr u -- )       \ RS232 Rom string to RS232
    R16 X+ LD,
    ZH X+ LD,
    ZL X+ LD,
    R16 TST,
    NZR IF,
        DO,     BEGIN,
                    ADR UDRE SBIS,
                AGAIN,
                LPM,
                UDR R0 OUT,
                ZL 1 ADIW,
        R16 LOOP,
    THEN,
    RET,
END-CODE

\G Verstuur 'u' karakters vanaf RAM adres 'addr' via de RS232.
CODE RS232-RTYPE    ( addr u -- )       \ RS232 "rs232-ram-type" Ram string to RS232
    R16 X+ LD,
    ZH X+ LD,
    ZL X+ LD,
    R16 TST,
    NZR IF,
        DO,     BEGIN,
                    ADR UDRE SBIS,
                AGAIN,
                R17 Z+ LD,
                UDR R17 OUT,
        R16 LOOP,
    THEN,
    RET,
END-CODE

\G 'Flag' is true als een kar. via RS232 ontvangen is, anders false.
UCSRA 7 BIT-SFR RS232-KEY? ( -- flag )  \ RS232 Char present from RS232

\G Wacht tot een karakter 'char' via RS232 ontvangen is.
CODE RS232-KEY      ( -- char )         \ RS232 Read char from RS232
    BEGIN,
        ADR RS232-KEY? SBIS,
    AGAIN,
    R16 UDR IN,
    -X R16 ST,
    RET,
END-CODE

MACRO: KEY?      RS232-KEY? ;                   ( -- flag )
MACRO: KEY       RS232-KEY ;                    ( -- char )
MACRO: EMIT      RS232-EMIT ;                   ( char -- )
MACRO: TYPE      RS232-TYPE ;                   ( addr u -- )
MACRO: RTYPE     RS232-RTYPE ;                  ( addr u -- )
: CR        ^M RS232-EMIT  ^J RS232-EMIT ;      ( -- )
: SPACE     BL RS232-EMIT ;                     ( -- )
: SPACES    ?DUP IF  FOR  SPACE  NEXT  THEN ;   ( u -- )
: PAGE      ^L RS232-EMIT ;                     ( -- )
: BS        ^H RS232-EMIT SPACE ^H RS232-EMIT ; ( -- )

\ Eind
