\ ----------------------------------------------------------------------
\ LANGUAGE    : AVR ByteForth vsn 2.00
\ PROJECT     : Software RS232
\ DESCRIPTION : Half duplex compacte software RS232
\ CATEGORY    : Applicatie, afmeting: 192 bytes.
\ AUTHOR      : Willem Ouwerkerk, August 02, 2003
\ LAST CHANGE : Willem Ouwerkerk, August 02, 2003
\ ----------------------------------------------------------------------

DOC umentatie betreffende baudrates.

Let op !!!!!!

Niet alle baudrates zijn bruikbaar met het woord WAIT-HALFBIT de 
maximum vertraging is nu 255. Het is aan de programmeur om een 
afwijkende versie ervan te schrijven om vertragingen groter dan 255 
toe te staan. Het betekent b.v. dat 2400 baud met een 4 MHz kristal 
nu niet mogelijk is. Het is daarom dat 19k2 als default snelheid is
gekozen, die werkt voor elk ondersteund kristal

ENDDOC

1 CRYSTAL? [IF]

\ #66 CONSTANT BITRATE#             \ 2400 Baud at 1 MHz
\ #31 CONSTANT BITRATE#             \ 4800 Baud
\ #14 CONSTANT BITRATE#             \ 9600 Baud

\G Deze constante bepaalt de snelheid van de software RS232.
\G Er zijn versies voor 1, 2, 4, 8 en 16 MHz. De basissnelheid 
\G is 19K2 baud voor alle kristallen.
  #05 CONSTANT BITRATE#  ( -- u )   \ RS232s  19k2 Baud

\ #02 CONSTANT BITRATE#             \ 28k8 Baud

[THEN]  

2 CRYSTAL? [IF]

\ #135 CONSTANT BITRATE#            \ 2400 Baud at 2 MHz
\ #066 CONSTANT BITRATE#            \ 4800 Baud
\ #031 CONSTANT BITRATE#            \ 9600 Baud
  #014 CONSTANT BITRATE#            \ 19k2 Baud
\ #008 CONSTANT BITRATE#            \ 28k8 Baud
\ #002 CONSTANT BITRATE#            \ 57k6 Baud

[THEN]  

4 CRYSTAL? [IF]

\ #274 CONSTANT BITRATE#            \ 2400 Baud at 4 MHz
\ #135 CONSTANT BITRATE#            \ 4800 Baud
\ #066 CONSTANT BITRATE#            \ 9600 Baud
  #031 CONSTANT BITRATE#            \ 19k2 Baud
\ #019 CONSTANT BITRATE#            \ 28k8 Baud
\ #008 CONSTANT BITRATE#            \ 57k6 Baud
\ #002 CONSTANT BITRATE#            \ 115k Baud

[THEN]  

8 CRYSTAL? [IF]

\ #552 CONSTANT BITRATE#            \ 2400 Baud at 8 MHz
\ #274 CONSTANT BITRATE#            \ 4800 Baud
\ #135 CONSTANT BITRATE#            \ 9600 Baud
  #066 CONSTANT BITRATE#            \ 19k2 Baud
\ #042 CONSTANT BITRATE#            \ 28k8 Baud
\ #019 CONSTANT BITRATE#            \ 57k6 Baud
\ #008 CONSTANT BITRATE#            \ 115k Baud

[THEN]  

16 CRYSTAL? [IF]

\ 1107 CONSTANT BITRATE#            \ 2400 Baud at 16 MHz
\ #552 CONSTANT BITRATE#            \ 4800 Baud
\ #274 CONSTANT BITRATE#            \ 9600 Baud
  #135 CONSTANT BITRATE#            \ 19k2 Baud
\ #089 CONSTANT BITRATE#            \ 28k8 Baud
\ #042 CONSTANT BITRATE#            \ 57k6 Baud
\ #019 CONSTANT BITRATE#            \ 115k Baud

[THEN]  

  #01 CONSTANT STOPBITS#            \ Number of stopbits

PIND  0 BIT-SFR RXD                 \ Select RS232 input
DDRD  0 BIT-SFR RRXD                \ Richting van RXD
PORTD 1 BIT-SFR TXD                 \ Select RS232 output

CODE WAIT-HALFBIT ( -- )            \ Wait half bittime
    R18 BITRATE# LDI,
    DO,  R18 LOOP,
    RET,
END-CODE

\G Verzend het karakter 'char' via de RS232.
CODE RS232-EMIT ( char -- )         \ RS232s Char to RS232
    R16 X+ LD,
    R17 9 STOPBITS# + LDI,          \ 1 + 8 + S bits
    R16 COM,                        \ Invert char
    SEC,                            \ Startbit is false
    DO,
        CS IF,
            ADR TXD CBI,            \ Send zero
        ELSE,
            ADR TXD SBI,            \ Send one
            NOP,
        THEN,
        ' WAIT-HALFBIT GCALL,       \ Wait bittime
        ' WAIT-HALFBIT GCALL,
        R16 LSR,                    \ Get next bit
    R17 LOOP,                       \ Send all bits
    RET,
END-CODE

\G Wacht tot het karakter 'char' via RS232 ontvangen is.
CODE RS232-KEY  ( -- char )         \ RS232s read char from RS232
    R17 9 LDI,                      \ Read 8 databits & 1 stopbit
    BEGIN,  ADR RXD SBIC, AGAIN,    \ Wait for startbit
    ' WAIT-HALFBIT GCALL,
    BEGIN,
        ' WAIT-HALFBIT GCALL,       \ Wait bittime
        ' WAIT-HALFBIT GCALL,
        CLC,                        \ Read RX line to carry
        ADR RXD SBIC,
        SEC,
        R17 DEC,                    \ Return at stopbit
    NZR WHILE,
        R16 ROR,                    \ Shift RX into char
    REPEAT,
    -X R16 ST,
    RET,
END-CODE

\G Initialiseer de I/O-bits voor RS232.
: SETUP-RS232   ( -- )              \ RS232s Initialise RS232
    -1 SETDIR TXD                   \ TXD is output
\   CLEAR RRXD                      \ RXD is input
    SET TXD                         \ Start with TXD high
    ;

\G Verzend 'u' karakters vanaf ROM adres 'addr' via de RS232.
: TYPE          ( addr u -- )       \ RS232s type string from ROM
    FOR
        COUNT RS232-EMIT
    NEXT
    2DROP
    ;

\G Verzend 'u' karakters vanaf RAM adres 'addr' via de RS232.
: RTYPE         ( addr u -- )       \ RS232s "ram-type" type string from RAM
    FOR
        @+ RS232-EMIT
    NEXT
    2DROP
    ;


MACRO: KEY?      TRUE ;                         ( -- flag )
MACRO: KEY       RS232-KEY ;                    ( -- char )
MACRO: EMIT      RS232-EMIT ;                   ( char -- )
MACRO: TYPE      RS232-TYPE ;                   ( $addr u -- )
MACRO: RTYPE     RS232-RTYPE ;                  ( addr u -- )
: CR        ^M RS232-EMIT  ^J RS232-EMIT ;      ( -- )
: SPACE     BL RS232-EMIT ;                     ( -- )
: SPACES    ?DUP IF  FOR  SPACE  NEXT  THEN ;   ( u -- )
: PAGE      ^L RS232-EMIT ;                     ( -- )
: BS        ^H RS232-EMIT SPACE ^H RS232-EMIT ; ( -- )

                              ( Einde )