\ ==============================================================
\ [TITLE]     AVR ByteForth TRACER target library. 
\ [FILE]      TRACER1.FRT
\ [CATEGORY]  Library, 436 bytes
\ [VSN]       1.00 
\ [CREATED]   October 10, 1997, Willem Ouwerkerk 
\ [LASTCHNGD] May 20, 2003, Willem Ouwerkerk
\ [COPYRIGHT] De Elektrise Schaar (C) 1997-2003
\ [USES]      AVR ByteForth cross compiler
\ [PURPOSE]   Tracer through RS232 with use baudrate generator
\ ==============================================================
\ [TITLE]     AVR ByteForth RS232 target library. 
\ ==============================================================

4 CRYSTAL?  [IF]

\ All baudrates mentioned are for a 4 MHz crystal

\ Initialiseer de ACIA op een RS232 baudrate van 9600 baud.
CODE SETUP-RS232                        \ RS232
\   R16 103 LDI,                        \ Select 2400 baud
\   R16 51 LDI,                         \ Select 4800 baud
    R16 25 LDI,                         \ Select 9600 baud
    UBRR R16 OUT,
    R16 $18 LDI,
    UCR R16 OUT,                        \ Enable receiver & transmitter
    RET,                                \ Reset interrupt, set 8 bit data.
END-CODE

[THEN]

8 CRYSTAL?  [IF]

\ All baudrates mentioned are for a 8 MHz crystal, for
\ a 4 MHz crystal the baudrates are 4800 or 9600 baud.

CODE SETUP-RS232
\   R16 103 LDI,                        \ Select 4800 baud
    R16 51 LDI,                         \ Select 9600 baud
    UBRR R16 OUT,
    R16 $18 LDI,
    UCR R16 OUT,                        \ Enable receiver & transmitter
    RET,                                \ Reset interrupt, set 8 bit data.
END-CODE

[THEN]

11 CRYSTAL? [IF]

\ All baudrates mentioned are for a 11.059 MHz crystal.
\ The max. baudrate is 115200 with this crystal.

CODE SETUP-RS232
\   R16 143 LDI,                        \ Select 4800 baud
    R16 71 LDI,                         \ Select 9600 baud
    UBRR R16 OUT,
    R16 $18 LDI,
    UCR R16 OUT,                        \ Enable receiver & transmitter
    RET,                                \ Reset interrupt, set 8 bit data.
END-CODE

[THEN]

\ Verzend het karakter char. via de RS232.
CODE EMIT           ( char -- )         \ RS232 Char to RS232
    R16 X+ LD,
    BEGIN,
        USR 5 SBIS,
    AGAIN,
    UDR R16 OUT,
    RET,
END-CODE

\ Verzend u karakters vanaf ROM adres $addr via de RS232.
CODE TYPE           ( $addr u -- )      \ RS232 Rom string to RS232
    R16 X+ LD,
    ZH X+ LD,
    ZL X+ LD,
    R16 TST,
    NZR IF,
        DO,     BEGIN,
                    USR 5 SBIS,
                AGAIN,
                LPM,
                UDR R0 OUT,
                ZL 1 ADIW,
        R16 LOOP,
    THEN,
    RET,
END-CODE

\ Verstuur u karakters vanaf RAM adres addr via de RS232.
CODE RTYPE          ( $addr u -- )      \ RS232 Ram string to RS232
    R16 X+ LD,
    ZH X+ LD,
    ZL X+ LD,
    R16 TST,
    NZR IF,
        DO,     BEGIN,
                    USR 5 SBIS,
                AGAIN,
                R17 Z+ LD,
                UDR R17 OUT,
        R16 LOOP,
    THEN,
    RET,
END-CODE

\ Wacht tot een karakter char via RS232 ontvangen is.
CODE KEY            ( -- char )         \ RS232 Read char from RS232
    BEGIN,
        USR 7 SBIS,
    AGAIN,
    R16 UDR IN,
    -X R16 ST,
    RET,
END-CODE

: CR        ^M EMIT  ^J EMIT ;      ( -- )
: SPACE     BL EMIT ;               ( -- )


\ ==============================================================
\ [TITLE]     AVR ByteForth number output library. 
\ ==============================================================

ATOM INLINE$

\ addr is het adres van de cel die het grondtal voor de getal
\ conversie bevat, geldige waarden zijn 2 t/m 36.
VARIABLE BASE      ( -- addr )          \ Numbers Hold number conversion base

VARIABLE HLD                            \ Work addr. in number conv. buffer
#10 CONSTANT NUM                        \ Length of number conversion buffer
NUM VARIABLES CONV                      \ Number string buffer
ADR CONV 2CONSTANT NUMSTART             \ Start of string buffer

\ Unsigned devide routine

\ Deel ud1 door het getal in BASE tot ud2 en char. Wordt gebruikt
\ om een digit te reduceren van een unsigned dubbel getal tot zijn
\ equivalent ASCII karakter.
CODE DIG-OUT ( ud1 -- ud2 char )        \ Numbers "dig-out" Divide ud1 by [BASE] to ud2 with char
    R16 X+ LD,
    R17 X+ LD,
    R18 CLR,
    R19 ADR BASE [Y] LD,                \ Base is divisor
    R20 16 LDI,                         \ 16 bits shift
    DO,
        SEC,                            \ Shift in a one
        R17 ROL,                        \ Shift lowbyte
        R16 ROL,                        \ Shift highbyte
        R18 ROL,                        \ Save highbit in workregister
        R18 R19 SBC,                    \ Subtract base
        CS IF,                          \ Unsuccessfull ?
            R17 DEC,                    \ Yes, correct one
            R18 R19 ADD,                \ Restore work register
        THEN,
    R20 LOOP,
    R18 11 CPI,                         \ Digit 10 U> ?
    UGE IF,
        R18 7 ADDI,                     \ Yes, correct 10 to "A"
    THEN,
    R18 &0 ADDI,                        \ Convert to ASCII
    -X R17 ST,
    -X R16 ST,
    -X R18 ST,
    RET,
END-CODE

\ Number conversion primitives

: HOLD      DECR HLD  FROM HLD TO CONV ;                    ( char -- )
: SIGN      0< IF  [CHAR] -  HOLD  THEN ;                   ( n -- )
: <#        NUM TO HLD ;                                    ( -- )
: #>        2DROP  NUMSTART FROM HLD M+  NUM FROM HLD - ;   ( d -- addr u )
: #         DIG-OUT HOLD ;                                  ( d1 -- d2 )
: #S        BEGIN  # 2DUP OR 0= UNTIL ;                     ( d1 -- d2 )

\ Number output words

: U.        0 <# #S  #>  RTYPE SPACE ;                      ( d -- )

: DECIMAL   #10 TO BASE ;                                   ( -- )


\ Tracer routines

ASSEMBLER
16   SFR ACCU
SREG SFR PSW
SPL  SFR SP
WORK

\G Toon de belangrijkste registers van de AVR.
: .REGISTERS    ( -- )                  \ Tracer "dot-registers" Show most important registers
    SP  PSW  ACCU  CR                   \ Read registers
    ." Accu: "      U.                  \ Accumulator
    ." Statusreg: " U.                  \ Program Status Word
    ." Stackp: "    U.                  \ Hardware Stackpointer
    ;

\G Copieer en toon alle gegevens op de datastack.
: .S            ( -- )                  \ Tracer "dot-s" Show stack picture
   ." ( "
    DEPTH IF
        DEPTH FOR
            I' 1- PICK U.
        NEXT
   THEN
   ." ) "
   ;

\G Zet een breakpunt, toon de registers en druk de stack af,
\G wacht daarna op een toetsaanslag.
: BREAKPOINT    ( -- )                  \ Tracer Show tracing info
    .REGISTERS                          \ Importand registers
    CR .S                               \ the datastack
    KEY DROP                            \ and wait for a key
    ;

\G Configureer de tracer en zet een breakpunt.
: TRACER-SETUP  ( -- )                  \ Tracer Setup for higlevel tracer
    SETUP-RS232
    DECIMAL
    BREAKPOINT
    ;

\ End
